@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018 Yusuke Suzuki <yusukesuzuki@slowstart.org>.
+ * Copyright (C) 2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,26 +26,32 @@
 
 #pragma once
 
-#if ENABLE(ASSEMBLER) && OS(LINUX)
+#include <wtf/Platform.h>
 
+#if ENABLE(ASSEMBLER)
+
+#include "LinkBuffer.h"
 #include <stdio.h>
 #include <wtf/Lock.h>
+#include <wtf/NeverDestroyed.h>
+#include <wtf/TZoneMalloc.h>
 #include <wtf/text/CString.h>
 
 namespace JSC {
 
 class PerfLog {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED(PerfLog);
     WTF_MAKE_NONCOPYABLE(PerfLog);
+    friend class LazyNeverDestroyed<PerfLog>;
 public:
-    static void log(CString&&, const uint8_t* executableAddress, size_t);
+    static void log(const CString& name, MacroAssemblerCodeRef<LinkBufferPtrTag>);
 
 private:
     PerfLog();
     static PerfLog& singleton();
 
-    void write(const void*, size_t) WTF_REQUIRES_LOCK(m_lock);
-    void flush() WTF_REQUIRES_LOCK(m_lock);
+    void write(const AbstractLocker&, const void*, size_t) WTF_REQUIRES_LOCK(m_lock);
+    void flush(const AbstractLocker&) WTF_REQUIRES_LOCK(m_lock);
 
     FILE* m_file { nullptr };
     void* m_marker { nullptr };
@@ -55,4 +62,4 @@ private:
 
 } // namespace JSC
 
-#endif // ENABLE(ASSEMBLER) && OS(LINUX)
+#endif // ENABLE(ASSEMBLER)
