@@ -29,18 +29,110 @@
 #include "Error.h"
 #include "JSArrayInlines.h"
 #include "JSFunction.h"
+#include "JSGenericTypedArrayViewInlines.h"
+#include "JSGlobalProxy.h"
 #include "JSObject.h"
 #include "JSTypedArrays.h"
 #include "Lookup.h"
+#include "MegamorphicCache.h"
+#include "ObjectInitializationScope.h"
+#include "SparseArrayValueMap.h"
 #include "StructureInlines.h"
 #include "TypedArrayType.h"
+#include "VM.h"
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
 
 namespace JSC {
 
+inline Structure* JSObject::createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype)
+{
+    return Structure::create(vm, globalObject, prototype, TypeInfo(ObjectType, StructureFlags), info());
+}
+
+inline Structure* JSNonFinalObject::createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype)
+{
+    return Structure::create(vm, globalObject, prototype, TypeInfo(ObjectType, StructureFlags), info());
+}
+
+inline Structure* JSFinalObject::createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype, unsigned inlineCapacity)
+{
+    return Structure::create(vm, globalObject, prototype, typeInfo(), info(), defaultIndexingType, inlineCapacity);
+}
+
+inline void JSObject::setButterfly(VM& vm, Butterfly* butterfly)
+{
+    if (isX86() || vm.heap.mutatorShouldBeFenced()) {
+        WTF::storeStoreFence();
+        m_butterfly.set(vm, this, butterfly);
+        WTF::storeStoreFence();
+        return;
+    }
+
+    m_butterfly.set(vm, this, butterfly);
+}
+
+inline void JSObject::nukeStructureAndSetButterfly(VM& vm, StructureID oldStructureID, Butterfly* butterfly)
+{
+    if (isX86() || vm.heap.mutatorShouldBeFenced()) {
+        setStructureIDDirectly(oldStructureID.nuke());
+        WTF::storeStoreFence();
+        m_butterfly.set(vm, this, butterfly);
+        WTF::storeStoreFence();
+        return;
+    }
+
+    m_butterfly.set(vm, this, butterfly);
+}
+
+inline JSValue JSObject::get(JSGlobalObject* globalObject, PropertyName propertyName) const
+{
+    VM& vm = getVM(globalObject);
+    auto scope = DECLARE_THROW_SCOPE(vm);
+    PropertySlot slot(this, PropertySlot::InternalMethodType::Get);
+    bool hasProperty = const_cast<JSObject*>(this)->getPropertySlot(globalObject, propertyName, slot);
+
+    EXCEPTION_ASSERT(!scope.exception() || vm.hasPendingTerminationException() || !hasProperty);
+    RETURN_IF_EXCEPTION(scope, jsUndefined());
+
+    if (hasProperty)
+        RELEASE_AND_RETURN(scope, slot.getValue(globalObject, propertyName));
+
+    return jsUndefined();
+}
+
+inline JSValue JSObject::get(JSGlobalObject* globalObject, unsigned propertyName) const
+{
+    VM& vm = getVM(globalObject);
+    auto scope = DECLARE_THROW_SCOPE(vm);
+    PropertySlot slot(this, PropertySlot::InternalMethodType::Get);
+    bool hasProperty = const_cast<JSObject*>(this)->getPropertySlot(globalObject, propertyName, slot);
+
+    EXCEPTION_ASSERT(!scope.exception() || vm.hasPendingTerminationException() || !hasProperty);
+    RETURN_IF_EXCEPTION(scope, jsUndefined());
+
+    if (hasProperty)
+        RELEASE_AND_RETURN(scope, slot.getValue(globalObject, propertyName));
+
+    return jsUndefined();
+}
+
+template<typename T, typename PropertyNameType>
+inline T JSObject::getAs(JSGlobalObject* globalObject, PropertyNameType propertyName) const
+{
+    JSValue value = get(globalObject, propertyName);
+#if ASSERT_ENABLED || ENABLE(SECURITY_ASSERTIONS)
+    VM& vm = getVM(globalObject);
+    if (vm.exceptionForInspection())
+        return nullptr;
+#endif
+    return jsCast<T>(value);
+}
+
 template<typename CellType, SubspaceAccess>
 CompleteSubspace* JSFinalObject::subspaceFor(VM& vm)
 {
-    static_assert(!CellType::needsDestruction);
+    static_assert(CellType::needsDestruction == DoesNotNeedDestruction);
     return &vm.cellSpace();
 }
 
@@ -50,7 +142,7 @@ void forEachInArrayLike(JSGlobalObject* globalObject, JSObject* arrayLikeObject,
 {
     VM& vm = getVM(globalObject);
     auto scope = DECLARE_THROW_SCOPE(vm);
-    uint64_t length = static_cast<uint64_t>(toLength(globalObject, arrayLikeObject));
+    uint64_t length = toLength(globalObject, arrayLikeObject);
     RETURN_IF_EXCEPTION(scope, void());
     for (uint64_t index = 0; index < length; index++) {
         JSValue value = arrayLikeObject->getIndex(globalObject, index);
@@ -84,7 +176,7 @@ ALWAYS_INLINE bool JSObject::canPerformFastPutInlineExcludingProto()
 
 ALWAYS_INLINE bool JSObject::canPerformFastPutInline(VM& vm, PropertyName propertyName)
 {
-    if (UNLIKELY(propertyName == vm.propertyNames->underscoreProto))
+    if (propertyName == vm.propertyNames->underscoreProto) [[unlikely]]
         return false;
     return canPerformFastPutInlineExcludingProto();
 }
@@ -117,17 +209,17 @@ ALWAYS_INLINE bool JSObject::getPropertySlot(JSGlobalObject* globalObject, unsig
         RETURN_IF_EXCEPTION(scope, false);
         if (hasSlot)
             return true;
-        if (UNLIKELY(slot.isVMInquiry() && slot.isTaintedByOpaqueObject()))
+        if (slot.isVMInquiry() && slot.isTaintedByOpaqueObject()) [[unlikely]]
             return false;
         if (object->type() == ProxyObjectType && slot.internalMethodType() == PropertySlot::InternalMethodType::HasProperty)
             return false;
         if (isTypedArrayType(object->type()) && propertyName >= jsCast<JSArrayBufferView*>(object)->length())
             return false;
         JSValue prototype;
-        if (LIKELY(!structure->typeInfo().overridesGetPrototype() || slot.internalMethodType() == PropertySlot::InternalMethodType::VMInquiry))
+        if (!structure->typeInfo().overridesGetPrototype() || slot.internalMethodType() == PropertySlot::InternalMethodType::VMInquiry) [[likely]]
             prototype = object->getPrototypeDirect();
         else {
-            prototype = object->getPrototype(vm, globalObject);
+            prototype = object->getPrototype(globalObject);
             RETURN_IF_EXCEPTION(scope, false);
         }
         if (!prototype.isObject())
@@ -138,7 +230,7 @@ ALWAYS_INLINE bool JSObject::getPropertySlot(JSGlobalObject* globalObject, unsig
 
 ALWAYS_INLINE bool JSObject::getPropertySlot(JSGlobalObject* globalObject, uint64_t propertyName, PropertySlot& slot)
 {
-    if (LIKELY(propertyName <= MAX_ARRAY_INDEX))
+    if (propertyName <= MAX_ARRAY_INDEX) [[likely]]
         return getPropertySlot(globalObject, static_cast<uint32_t>(propertyName), slot);
     return getPropertySlot(globalObject, Identifier::from(globalObject->vm(), propertyName), slot);
 }
@@ -153,7 +245,7 @@ ALWAYS_INLINE bool JSObject::getNonIndexPropertySlot(JSGlobalObject* globalObjec
     JSObject* object = this;
     while (true) {
         Structure* structure = object->structureID().decode();
-        if (LIKELY(!TypeInfo::overridesGetOwnPropertySlot(object->inlineTypeFlags()))) {
+        if (!TypeInfo::overridesGetOwnPropertySlot(object->inlineTypeFlags())) [[likely]] {
             if (object->getOwnNonIndexPropertySlot(vm, structure, propertyName, slot))
                 return true;
         } else {
@@ -161,18 +253,18 @@ ALWAYS_INLINE bool JSObject::getNonIndexPropertySlot(JSGlobalObject* globalObjec
             RETURN_IF_EXCEPTION(scope, false);
             if (hasSlot)
                 return true;
-            if (UNLIKELY(slot.isVMInquiry() && slot.isTaintedByOpaqueObject()))
+            if (slot.isVMInquiry() && slot.isTaintedByOpaqueObject()) [[unlikely]]
                 return false;
             if (object->type() == ProxyObjectType && slot.internalMethodType() == PropertySlot::InternalMethodType::HasProperty)
                 return false;
-            if (isTypedArrayType(object->type()) && isCanonicalNumericIndexString(propertyName))
+            if (isTypedArrayType(object->type()) && isCanonicalNumericIndexString(propertyName.uid()))
                 return false;
         }
         JSValue prototype;
-        if (LIKELY(!structure->typeInfo().overridesGetPrototype() || slot.internalMethodType() == PropertySlot::InternalMethodType::VMInquiry))
+        if (!structure->typeInfo().overridesGetPrototype() || slot.internalMethodType() == PropertySlot::InternalMethodType::VMInquiry) [[likely]]
             prototype = object->getPrototypeDirect();
         else {
-            prototype = object->getPrototype(vm, globalObject);
+            prototype = object->getPrototype(globalObject);
             RETURN_IF_EXCEPTION(scope, false);
         }
         if (!prototype.isObject())
@@ -183,7 +275,7 @@ ALWAYS_INLINE bool JSObject::getNonIndexPropertySlot(JSGlobalObject* globalObjec
 
 inline bool JSObject::getOwnPropertySlotInline(JSGlobalObject* globalObject, PropertyName propertyName, PropertySlot& slot)
 {
-    if (UNLIKELY(TypeInfo::overridesGetOwnPropertySlot(inlineTypeFlags())))
+    if (TypeInfo::overridesGetOwnPropertySlot(inlineTypeFlags())) [[unlikely]]
         return methodTable()->getOwnPropertySlot(this, globalObject, propertyName, slot);
     return JSObject::getOwnPropertySlot(this, globalObject, propertyName, slot);
 }
@@ -200,7 +292,7 @@ template<typename PropertyNameType> inline JSValue JSObject::getIfPropertyExists
         return { };
 
     scope.release();
-    if (UNLIKELY(slot.isTaintedByOpaqueObject()))
+    if (slot.isTaintedByOpaqueObject()) [[unlikely]]
         return get(globalObject, propertyName);
 
     return slot.getValue(globalObject, propertyName);
@@ -214,19 +306,19 @@ inline bool JSObject::noSideEffectMayHaveNonIndexProperty(VM& vm, PropertyName p
     ASSERT(propertyName != vm.propertyNames->length);
     for (auto* object = this; object; object = object->getPrototypeDirect().getObject()) {
         auto inlineTypeFlags = object->inlineTypeFlags();
-        if (UNLIKELY(TypeInfo::overridesGetOwnPropertySlot(inlineTypeFlags) && object->classInfo() != ArrayPrototype::info()))
+        if (TypeInfo::overridesGetOwnPropertySlot(inlineTypeFlags) && object->classInfo() != ArrayPrototype::info()) [[unlikely]]
             return true;
         auto& structure = *object->structureID().decode();
         unsigned attributes;
-        if (UNLIKELY(isValidOffset(structure.get(vm, propertyName, attributes))))
+        if (isValidOffset(structure.get(vm, propertyName, attributes))) [[unlikely]]
             return true;
-        if (TypeInfo::hasStaticPropertyTable(inlineTypeFlags) && !structure.staticPropertiesReified()) {
+        if (hasNonReifiedStaticProperties()) {
             for (auto* ancestorClass = object->classInfo(); ancestorClass; ancestorClass = ancestorClass->parentClass) {
-                if (auto* table = ancestorClass->staticPropHashTable; UNLIKELY(table && table->entry(propertyName)))
+                if (auto* table = ancestorClass->staticPropHashTable; table && table->entry(propertyName)) [[unlikely]]
                     return true;
             }
         }
-        if (UNLIKELY(structure.typeInfo().overridesGetPrototype()))
+        if (structure.typeInfo().overridesGetPrototype()) [[unlikely]]
             return true;
     }
     return false;
@@ -271,6 +363,8 @@ ALWAYS_INLINE PropertyOffset JSObject::prepareToPutDirectWithoutTransition(VM& v
             ASSERT(!getDirect(offset) || !JSValue::encode(getDirect(offset)));
             result = offset;
         });
+    if (mayBePrototype()) [[unlikely]]
+        vm.invalidateStructureChainIntegrity(VM::StructureChainIntegrityEvent::Add);
     return result;
 }
 
@@ -286,16 +380,16 @@ ALWAYS_INLINE bool JSObject::putInlineForJSObject(JSCell* cell, JSGlobalObject*
     // Try indexed put first. This is required for correctness, since loads on property names that appear like
     // valid indices will never look in the named property storage.
     if (std::optional<uint32_t> index = parseIndex(propertyName)) {
-        if (UNLIKELY(isThisValueAltered(slot, thisObject)))
+        if (isThisValueAltered(slot, thisObject)) [[unlikely]]
             return ordinarySetSlow(globalObject, thisObject, propertyName, value, slot.thisValue(), slot.isStrictMode());
         return thisObject->methodTable()->putByIndex(thisObject, globalObject, index.value(), value, slot.isStrictMode());
     }
 
     if (!thisObject->canPerformFastPutInline(vm, propertyName))
         return thisObject->putInlineSlow(globalObject, propertyName, value, slot);
-    if (UNLIKELY(isThisValueAltered(slot, thisObject)))
+    if (isThisValueAltered(slot, thisObject)) [[unlikely]]
         return definePropertyOnReceiver(globalObject, propertyName, value, slot);
-    if (UNLIKELY(thisObject->hasNonReifiedStaticProperties()))
+    if (thisObject->hasNonReifiedStaticProperties()) [[unlikely]]
         return thisObject->putInlineFastReplacingStaticPropertyIfNeeded(globalObject, propertyName, value, slot);
     return thisObject->putInlineFast(globalObject, propertyName, value, slot);
 }
@@ -323,7 +417,7 @@ ALWAYS_INLINE bool JSObject::createDataProperty(JSGlobalObject* globalObject, Pr
 ALWAYS_INLINE bool JSObject::hasOwnProperty(JSGlobalObject* globalObject, PropertyName propertyName, PropertySlot& slot) const
 {
     ASSERT(slot.internalMethodType() == PropertySlot::InternalMethodType::GetOwnProperty);
-    if (LIKELY(const_cast<JSObject*>(this)->methodTable()->getOwnPropertySlot == JSObject::getOwnPropertySlot))
+    if (const_cast<JSObject*>(this)->methodTable()->getOwnPropertySlot == JSObject::getOwnPropertySlot) [[likely]]
         return JSObject::getOwnPropertySlot(const_cast<JSObject*>(this), globalObject, propertyName, slot);
     return const_cast<JSObject*>(this)->methodTable()->getOwnPropertySlot(const_cast<JSObject*>(this), globalObject, propertyName, slot);
 }
@@ -341,11 +435,11 @@ ALWAYS_INLINE bool JSObject::hasOwnProperty(JSGlobalObject* globalObject, unsign
 }
 
 template<JSObject::PutMode mode>
-ALWAYS_INLINE ASCIILiteral JSObject::putDirectInternal(VM& vm, PropertyName propertyName, JSValue value, unsigned attributes, PutPropertySlot& slot)
+ALWAYS_INLINE ASCIILiteral JSObject::putDirectInternal(VM& vm, PropertyName propertyName, JSValue value, unsigned newAttributes, PutPropertySlot& slot)
 {
     ASSERT(value);
-    ASSERT(value.isGetterSetter() == !!(attributes & PropertyAttribute::Accessor));
-    ASSERT(value.isCustomGetterSetter() == !!(attributes & PropertyAttribute::CustomAccessorOrValue));
+    ASSERT(value.isGetterSetter() == !!(newAttributes & PropertyAttribute::Accessor));
+    ASSERT(value.isCustomGetterSetter() == !!(newAttributes & PropertyAttribute::CustomAccessorOrValue));
     ASSERT(!Heap::heap(value) || Heap::heap(value) == Heap::heap(this));
     ASSERT(!parseIndex(propertyName));
 
@@ -353,66 +447,89 @@ ALWAYS_INLINE ASCIILiteral JSObject::putDirectInternal(VM& vm, PropertyName prop
     Structure* structure = structureID.decode();
     if (structure->isDictionary()) {
         ASSERT(!isCopyOnWrite(indexingMode()));
-        
-        unsigned currentAttributes;
-        PropertyOffset offset = structure->get(vm, propertyName, currentAttributes);
-        if (offset != invalidOffset) {
-            if (mode == PutModePut && (currentAttributes & PropertyAttribute::ReadOnlyOrAccessorOrCustomAccessor))
-                return ReadonlyPropertyChangeError;
+        if constexpr (mode == PutModePut) {
+            if (!isStructureExtensible()) [[unlikely]]
+                return putDirectToDictionaryWithoutExtensibility(vm, propertyName, value, slot);
+        }
+
+        auto [offset, attributes, isAdded] = structure->addOrReplacePropertyWithoutTransition(vm, propertyName, newAttributes, [&](const GCSafeConcurrentJSLocker&, PropertyOffset offset, PropertyOffset newMaxOffset) {
+            unsigned oldOutOfLineCapacity = structure->outOfLineCapacity();
+            unsigned newOutOfLineCapacity = Structure::outOfLineCapacity(newMaxOffset);
+            if (newOutOfLineCapacity != oldOutOfLineCapacity) {
+                Butterfly* butterfly = allocateMoreOutOfLineStorage(vm, oldOutOfLineCapacity, newOutOfLineCapacity);
+                nukeStructureAndSetButterfly(vm, structureID, butterfly);
+                structure->setMaxOffset(vm, newMaxOffset);
+                WTF::storeStoreFence();
+                setStructureIDDirectly(structureID);
+            } else
+                structure->setMaxOffset(vm, newMaxOffset);
+
+            // This assertion verifies that the concurrent GC won't read garbage if the concurrentGC
+            // is running at the same time we put without transitioning.
+            ASSERT_UNUSED(offset, !getDirect(offset) || !JSValue::encode(getDirect(offset)));
+        });
+
+        if (!isAdded) {
+            if constexpr (mode == PutModePut) {
+                if (attributes & PropertyAttribute::ReadOnlyOrAccessorOrCustomAccessor) [[unlikely]]
+                    return ReadonlyPropertyChangeError;
+            }
 
             putDirectOffset(vm, offset, value);
             structure->didReplaceProperty(offset);
 
             // FIXME: Check attributes against PropertyAttribute::CustomAccessorOrValue. Changing GetterSetter should work w/o transition.
             // https://bugs.webkit.org/show_bug.cgi?id=214342
-            if (mode == PutModeDefineOwnProperty && (attributes != currentAttributes || (attributes & PropertyAttribute::AccessorOrCustomAccessorOrValue)))
-                setStructure(vm, Structure::attributeChangeTransition(vm, structure, propertyName, attributes));
-            else {
-                ASSERT(!(currentAttributes & PropertyAttribute::AccessorOrCustomAccessorOrValue));
+            if ((mode == PutModeDefineOwnProperty) && (newAttributes != attributes || (newAttributes & PropertyAttribute::AccessorOrCustomAccessorOrValue))) {
+                DeferredStructureTransitionWatchpointFire deferred(vm, structure);
+                setStructure(vm, Structure::attributeChangeTransition(vm, structure, propertyName, newAttributes, &deferred));
+                if (mayBePrototype()) [[unlikely]]
+                    vm.invalidateStructureChainIntegrity(VM::StructureChainIntegrityEvent::Change);
+            } else {
+                ASSERT(!(attributes & PropertyAttribute::AccessorOrCustomAccessorOrValue));
                 slot.setExistingProperty(this, offset);
             }
-
             return { };
         }
 
-        if (mode == PutModePut && !isStructureExtensible())
-            return NonExtensibleObjectPropertyDefineError;
-
-        offset = prepareToPutDirectWithoutTransition(vm, propertyName, attributes, structureID, structure);
         validateOffset(offset);
         putDirectOffset(vm, offset, value);
         slot.setNewProperty(this, offset);
         if (attributes & PropertyAttribute::ReadOnly)
             this->structure()->setContainsReadOnlyProperties();
+        if (mayBePrototype()) [[unlikely]]
+            vm.invalidateStructureChainIntegrity(VM::StructureChainIntegrityEvent::Add);
         return { };
     }
 
-    PropertyOffset offset;
-    size_t currentCapacity = this->structure()->outOfLineCapacity();
-    Structure* newStructure = Structure::addPropertyTransitionToExistingStructure(
-        structure, propertyName, attributes, offset);
-    if (newStructure) {
-        Butterfly* newButterfly = butterfly();
-        if (currentCapacity != newStructure->outOfLineCapacity()) {
-            ASSERT(newStructure != this->structure());
-            newButterfly = allocateMoreOutOfLineStorage(vm, currentCapacity, newStructure->outOfLineCapacity());
-            nukeStructureAndSetButterfly(vm, structureID, newButterfly);
+    {
+        PropertyOffset offset;
+        Structure* newStructure = Structure::addPropertyTransitionToExistingStructure(structure, propertyName, newAttributes, offset);
+        if (newStructure) {
+            Butterfly* newButterfly = butterfly();
+            if (structure->outOfLineCapacity() != newStructure->outOfLineCapacity()) {
+                ASSERT(newStructure != this->structure());
+                newButterfly = allocateMoreOutOfLineStorage(vm, structure->outOfLineCapacity(), newStructure->outOfLineCapacity());
+                nukeStructureAndSetButterfly(vm, structureID, newButterfly);
+            }
+
+            validateOffset(offset);
+            ASSERT(newStructure->isValidOffset(offset));
+
+            // This assertion verifies that the concurrent GC won't read garbage if the concurrentGC
+            // is running at the same time we put without transitioning.
+            ASSERT(!getDirect(offset) || !JSValue::encode(getDirect(offset)));
+            putDirectOffset(vm, offset, value);
+            setStructure(vm, newStructure);
+            slot.setNewProperty(this, offset);
+            if (mayBePrototype()) [[unlikely]]
+                vm.invalidateStructureChainIntegrity(VM::StructureChainIntegrityEvent::Add);
+            return { };
         }
-
-        validateOffset(offset);
-        ASSERT(newStructure->isValidOffset(offset));
-
-        // This assertion verifies that the concurrent GC won't read garbage if the concurrentGC
-        // is running at the same time we put without transitioning.
-        ASSERT(!getDirect(offset) || !JSValue::encode(getDirect(offset)));
-        putDirectOffset(vm, offset, value);
-        setStructure(vm, newStructure);
-        slot.setNewProperty(this, offset);
-        return { };
     }
 
     unsigned currentAttributes;
-    offset = structure->get(vm, propertyName, currentAttributes);
+    PropertyOffset offset = structure->get(vm, propertyName, currentAttributes);
     if (offset != invalidOffset) {
         if (mode == PutModePut && (currentAttributes & PropertyAttribute::ReadOnlyOrAccessorOrCustomAccessor))
             return ReadonlyPropertyChangeError;
@@ -422,11 +539,13 @@ ALWAYS_INLINE ASCIILiteral JSObject::putDirectInternal(VM& vm, PropertyName prop
 
         // FIXME: Check attributes against PropertyAttribute::CustomAccessorOrValue. Changing GetterSetter should work w/o transition.
         // https://bugs.webkit.org/show_bug.cgi?id=214342
-        if (mode == PutModeDefineOwnProperty && (attributes != currentAttributes || (attributes & PropertyAttribute::AccessorOrCustomAccessorOrValue))) {
+        if ((mode == PutModeDefineOwnProperty) && (newAttributes != currentAttributes || (newAttributes & PropertyAttribute::AccessorOrCustomAccessorOrValue))) {
             // We want the structure transition watchpoint to fire after this object has switched structure.
             // This allows adaptive watchpoints to observe if the new structure is the one we want.
             DeferredStructureTransitionWatchpointFire deferredWatchpointFire(vm, structure);
-            setStructure(vm, Structure::attributeChangeTransition(vm, structure, propertyName, attributes, &deferredWatchpointFire));
+            setStructure(vm, Structure::attributeChangeTransition(vm, structure, propertyName, newAttributes, &deferredWatchpointFire));
+            if (mayBePrototype()) [[unlikely]]
+                vm.invalidateStructureChainIntegrity(VM::StructureChainIntegrityEvent::Change);
         } else {
             ASSERT(!(currentAttributes & PropertyAttribute::AccessorOrCustomAccessorOrValue));
             slot.setExistingProperty(this, offset);
@@ -435,13 +554,15 @@ ALWAYS_INLINE ASCIILiteral JSObject::putDirectInternal(VM& vm, PropertyName prop
         return { };
     }
 
-    if (mode == PutModePut && !isStructureExtensible())
-        return NonExtensibleObjectPropertyDefineError;
+    if constexpr (mode == PutModePut) {
+        if (!isStructureExtensible()) [[unlikely]]
+            return NonExtensibleObjectPropertyDefineError;
+    }
     
     // We want the structure transition watchpoint to fire after this object has switched structure.
     // This allows adaptive watchpoints to observe if the new structure is the one we want.
     DeferredStructureTransitionWatchpointFire deferredWatchpointFire(vm, structure);
-    newStructure = Structure::addNewPropertyTransition(vm, structure, propertyName, attributes, offset, slot.context(), &deferredWatchpointFire);
+    Structure* newStructure = Structure::addNewPropertyTransition(vm, structure, propertyName, newAttributes, offset, slot.context(), &deferredWatchpointFire);
     
     validateOffset(offset);
     ASSERT(newStructure->isValidOffset(offset));
@@ -459,19 +580,28 @@ ALWAYS_INLINE ASCIILiteral JSObject::putDirectInternal(VM& vm, PropertyName prop
     putDirectOffset(vm, offset, value);
     setStructure(vm, newStructure);
     slot.setNewProperty(this, offset);
-    if (attributes & PropertyAttribute::ReadOnly)
+    if (newAttributes & PropertyAttribute::ReadOnly)
         newStructure->setContainsReadOnlyProperties();
+    if (mayBePrototype()) [[unlikely]]
+        vm.invalidateStructureChainIntegrity(VM::StructureChainIntegrityEvent::Add);
     return { };
 }
 
 inline bool JSObject::mayBePrototype() const
 {
-    return perCellBit();
+    return structure()->mayBePrototype();
 }
 
-inline void JSObject::didBecomePrototype()
+inline void JSObject::didBecomePrototype(VM& vm)
 {
-    setPerCellBit(true);
+    Structure* oldStructure = structure();
+    if (!oldStructure->mayBePrototype()) [[unlikely]] {
+        DeferredStructureTransitionWatchpointFire deferred(vm, oldStructure);
+        setStructure(vm, Structure::becomePrototypeTransition(vm, oldStructure, &deferred));
+    }
+
+    if (type() == GlobalProxyType) [[unlikely]]
+        jsCast<JSGlobalProxy*>(this)->target()->didBecomePrototype(vm);
 }
 
 inline bool JSObject::canGetIndexQuicklyForTypedArray(unsigned i) const
@@ -649,7 +779,7 @@ inline bool JSObject::deleteProperty(JSGlobalObject* globalObject, uint32_t prop
 
 inline bool JSObject::deleteProperty(JSGlobalObject* globalObject, uint64_t propertyName)
 {
-    if (LIKELY(propertyName <= MAX_ARRAY_INDEX))
+    if (propertyName <= MAX_ARRAY_INDEX) [[likely]]
         return deleteProperty(globalObject, static_cast<uint32_t>(propertyName));
     ASSERT(propertyName <= maxSafeInteger());
     return deleteProperty(globalObject, Identifier::from(globalObject->vm(), propertyName));
@@ -657,7 +787,7 @@ inline bool JSObject::deleteProperty(JSGlobalObject* globalObject, uint64_t prop
 
 inline JSValue JSObject::get(JSGlobalObject* globalObject, uint64_t propertyName) const
 {
-    if (LIKELY(propertyName <= MAX_ARRAY_INDEX))
+    if (propertyName <= MAX_ARRAY_INDEX) [[likely]]
         return get(globalObject, static_cast<uint32_t>(propertyName));
     ASSERT(propertyName <= maxSafeInteger());
     return get(globalObject, Identifier::from(globalObject->vm(), propertyName));
@@ -753,6 +883,7 @@ ALWAYS_INLINE void JSObject::getNonReifiedStaticPropertyNames(VM& vm, PropertyNa
     if (staticPropertiesReified())
         return;
 
+    Structure* structure = this->structure();
     // Add properties from the static hashtables of properties
     for (const ClassInfo* info = classInfo(); info; info = info->parentClass) {
         const HashTable* table = info->staticPropHashTable;
@@ -760,8 +891,15 @@ ALWAYS_INLINE void JSObject::getNonReifiedStaticPropertyNames(VM& vm, PropertyNa
             continue;
 
         for (auto iter = table->begin(); iter != table->end(); ++iter) {
-            if (mode == DontEnumPropertiesMode::Include || !(iter->attributes() & PropertyAttribute::DontEnum))
-                propertyNames.add(Identifier::fromLatin1(vm, iter.key()));
+            if (mode == DontEnumPropertiesMode::Include || !(iter->attributes() & PropertyAttribute::DontEnum)) {
+                auto identifier = Identifier::fromString(vm, iter.key());
+                // If the structure is shadowing the static property use it's attributes to determine if
+                // the property name is enumerable but add it here to preserve the right property order.
+                unsigned structureAttributes;
+                if (isValidOffset(structure->get(vm, identifier, structureAttributes)) && (mode == DontEnumPropertiesMode::Exclude && (structureAttributes & PropertyAttribute::DontEnum)))
+                    continue;
+                propertyNames.add(identifier);
+            }
         }
     }
 }
@@ -807,4 +945,251 @@ inline void JSObject::setPrivateBrand(JSGlobalObject* globalObject, JSValue bran
     this->setStructure(vm, newStructure);
 }
 
+// Function forEachOwnIndexedProperty should only used in the fast path
+// for copying own non-GetterSetter indexed properties.
+template<JSObject::SortMode mode, typename Functor>
+void JSObject::forEachOwnIndexedProperty(JSGlobalObject* globalObject, const Functor& functor)
+{
+    ASSERT(structure()->canPerformFastPropertyEnumerationCommon());
+    ASSERT(canHaveExistingOwnIndexedProperties() && !canHaveExistingOwnIndexedGetterSetterProperties());
+    VM& vm = globalObject->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    switch (indexingType()) {
+    case ALL_BLANK_INDEXING_TYPES:
+    case ALL_UNDECIDED_INDEXING_TYPES:
+        break;
+
+    case ALL_INT32_INDEXING_TYPES:
+    case ALL_CONTIGUOUS_INDEXING_TYPES:
+    case ALL_DOUBLE_INDEXING_TYPES: {
+        unsigned usedLength = m_butterfly->publicLength();
+        for (unsigned i = 0; i < usedLength; ++i) {
+            JSValue value = getDirectIndex(globalObject, i);
+            RETURN_IF_EXCEPTION(scope, void());
+            if (value && functor(i, value) == IterationStatus::Done)
+                return;
+        }
+        break;
+    }
+
+    case ALL_ARRAY_STORAGE_INDEXING_TYPES: {
+        ArrayStorage* storage = m_butterfly->arrayStorage();
+        unsigned usedVectorLength = std::min(storage->length(), storage->vectorLength());
+        for (unsigned i = 0; i < usedVectorLength; ++i) {
+            auto value = storage->m_vector[i];
+            if (!value)
+                continue;
+            if (functor(i, value.get()) == IterationStatus::Done)
+                return;
+        }
+
+        if (SparseArrayValueMap* map = storage->m_sparseMap.get()) {
+            MarkedArgumentBuffer values;
+            if constexpr (mode == JSObject::SortMode::Default) {
+                Vector<unsigned, 8> properties;
+                for (auto& [key, value] : *map) {
+                    if (!(value.attributes() & PropertyAttribute::DontEnum)) {
+                        properties.append(key);
+                        values.appendWithCrashOnOverflow(value.get());
+                    }
+                }
+
+                for (size_t i = 0; i < properties.size(); ++i) {
+                    if (functor(properties[i], values.at(i)) == IterationStatus::Done)
+                        return;
+                }
+            } else {
+                Vector<std::tuple<unsigned, unsigned>, 8> propertyAndValueIndexTuples;
+                unsigned valueIndex = 0;
+                for (auto& [key, value] : *map) {
+                    if (!(value.attributes() & PropertyAttribute::DontEnum)) {
+                        propertyAndValueIndexTuples.append({ key, valueIndex++ });
+                        values.appendWithCrashOnOverflow(value.get());
+                    }
+                }
+
+                std::sort(propertyAndValueIndexTuples.begin(), propertyAndValueIndexTuples.end(), [](auto a, auto b) {
+                    return std::get<0>(a) < std::get<0>(b);
+                });
+                for (size_t i = 0; i < propertyAndValueIndexTuples.size(); ++i) {
+                    auto [property, valueIndex] = propertyAndValueIndexTuples.at(i);
+                    if (functor(property, values.at(valueIndex)) == IterationStatus::Done)
+                        return;
+                }
+            }
+        }
+        break;
+    }
+
+    default:
+        RELEASE_ASSERT_NOT_REACHED();
+    }
+}
+
+inline void JSObject::initializeIndex(ObjectInitializationScope& scope, unsigned i, JSValue v)
+{
+    initializeIndex(scope, i, v, indexingType());
+}
+
+ALWAYS_INLINE void JSObject::initializeIndex(ObjectInitializationScope& scope, unsigned i, JSValue v, IndexingType indexingType)
+{
+    VM& vm = scope.vm();
+    Butterfly* butterfly = m_butterfly.get();
+    switch (indexingType) {
+    case ALL_UNDECIDED_INDEXING_TYPES: {
+        setIndexQuicklyToUndecided(vm, i, v);
+        break;
+    }
+    case ALL_INT32_INDEXING_TYPES: {
+        ASSERT(i < butterfly->publicLength());
+        ASSERT(i < butterfly->vectorLength());
+        if (!v.isInt32()) {
+            convertInt32ToDoubleOrContiguousWhilePerformingSetIndex(vm, i, v);
+            break;
+        }
+        [[fallthrough]];
+    }
+    case ALL_CONTIGUOUS_INDEXING_TYPES: {
+        ASSERT(i < butterfly->publicLength());
+        ASSERT(i < butterfly->vectorLength());
+        butterfly->contiguous().at(this, i).set(vm, this, v);
+        break;
+    }
+    case ALL_DOUBLE_INDEXING_TYPES: {
+        ASSERT(i < butterfly->publicLength());
+        ASSERT(i < butterfly->vectorLength());
+        if (!v.isNumber()) {
+            convertDoubleToContiguousWhilePerformingSetIndex(vm, i, v);
+            return;
+        }
+        double value = v.asNumber();
+        if (value != value) {
+            convertDoubleToContiguousWhilePerformingSetIndex(vm, i, v);
+            return;
+        }
+        butterfly->contiguousDouble().at(this, i) = value;
+        break;
+    }
+    case ALL_ARRAY_STORAGE_INDEXING_TYPES: {
+        ArrayStorage* storage = butterfly->arrayStorage();
+        ASSERT(i < storage->length());
+        ASSERT(i < storage->m_numValuesInVector);
+        storage->m_vector[i].set(vm, this, v);
+        break;
+    }
+    default:
+        RELEASE_ASSERT_NOT_REACHED();
+    }
+}
+
+inline void JSObject::initializeIndexWithoutBarrier(ObjectInitializationScope& scope, unsigned i, JSValue v)
+{
+    initializeIndexWithoutBarrier(scope, i, v, indexingType());
+}
+
+ALWAYS_INLINE void JSObject::initializeIndexWithoutBarrier(ObjectInitializationScope&, unsigned i, JSValue v, IndexingType indexingType)
+{
+    Butterfly* butterfly = m_butterfly.get();
+    switch (indexingType) {
+    case ALL_UNDECIDED_INDEXING_TYPES: {
+        RELEASE_ASSERT_NOT_REACHED();
+        break;
+    }
+    case ALL_INT32_INDEXING_TYPES: {
+        ASSERT(i < butterfly->publicLength());
+        ASSERT(i < butterfly->vectorLength());
+        RELEASE_ASSERT(v.isInt32());
+        [[fallthrough]];
+    }
+    case ALL_CONTIGUOUS_INDEXING_TYPES: {
+        ASSERT(i < butterfly->publicLength());
+        ASSERT(i < butterfly->vectorLength());
+        butterfly->contiguous().at(this, i).setWithoutWriteBarrier(v);
+        break;
+    }
+    case ALL_DOUBLE_INDEXING_TYPES: {
+        ASSERT(i < butterfly->publicLength());
+        ASSERT(i < butterfly->vectorLength());
+        RELEASE_ASSERT(v.isNumber());
+        double value = v.asNumber();
+        RELEASE_ASSERT(value == value);
+        butterfly->contiguousDouble().at(this, i) = value;
+        break;
+    }
+    case ALL_ARRAY_STORAGE_INDEXING_TYPES: {
+        ArrayStorage* storage = butterfly->arrayStorage();
+        ASSERT(i < storage->length());
+        ASSERT(i < storage->m_numValuesInVector);
+        storage->m_vector[i].setWithoutWriteBarrier(v);
+        break;
+    }
+    default:
+        RELEASE_ASSERT_NOT_REACHED();
+    }
+}
+
+inline bool JSObject::canHaveExistingOwnIndexedGetterSetterProperties()
+{
+    if (!hasIndexedProperties(indexingType()))
+        return false;
+
+    switch (indexingType()) {
+    case ALL_BLANK_INDEXING_TYPES:
+    case ALL_UNDECIDED_INDEXING_TYPES:
+    case ALL_INT32_INDEXING_TYPES:
+    case ALL_CONTIGUOUS_INDEXING_TYPES:
+    case ALL_DOUBLE_INDEXING_TYPES:
+        return false;
+    case ALL_ARRAY_STORAGE_INDEXING_TYPES: {
+        SparseArrayValueMap* map = m_butterfly->arrayStorage()->m_sparseMap.get();
+        if (!map)
+            return false;
+        return map->hasAnyKindOfGetterSetterProperties();
+    }
+    default:
+        RELEASE_ASSERT_NOT_REACHED();
+    }
+}
+
+inline unsigned JSObject::canHaveExistingOwnIndexedProperties() const
+{
+    if (!hasIndexedProperties(indexingType()))
+        return false;
+
+    switch (indexingType()) {
+    case ALL_BLANK_INDEXING_TYPES:
+    case ALL_UNDECIDED_INDEXING_TYPES:
+        return false;
+    case ALL_INT32_INDEXING_TYPES:
+    case ALL_CONTIGUOUS_INDEXING_TYPES:
+    case ALL_DOUBLE_INDEXING_TYPES:
+        return m_butterfly->publicLength();
+    case ALL_ARRAY_STORAGE_INDEXING_TYPES: {
+        ArrayStorage* storage = m_butterfly->arrayStorage();
+        unsigned usedVectorLength = std::min(storage->length(), storage->vectorLength());
+        if (usedVectorLength)
+            return true;
+        SparseArrayValueMap* map = storage->m_sparseMap.get();
+        if (!map)
+            return false;
+        return map->size();
+    }
+    default:
+        RELEASE_ASSERT_NOT_REACHED();
+    }
+}
+
+ALWAYS_INLINE JSFinalObject* JSFinalObject::createDefaultEmptyObject(JSGlobalObject* globalObject)
+{
+    VM& vm = getVM(globalObject);
+    JSFinalObject* finalObject = new (NotNull, allocateCell<JSFinalObject>(vm, allocationSize(defaultInlineCapacity))) JSFinalObject(CreatingWellDefinedBuiltinCell, globalObject->objectStructureIDForObjectConstructor());
+    finalObject->finishCreation(vm);
+    ASSERT(globalObject->objectStructureForObjectConstructor()->id() == globalObject->objectStructureIDForObjectConstructor());
+    ASSERT(globalObject->objectStructureForObjectConstructor()->inlineCapacity() == defaultInlineCapacity);
+    return finalObject;
+}
+
 } // namespace JSC
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
