@@ -30,13 +30,15 @@
 #include "StrongInlines.h"
 #include "StructureInlines.h"
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC {
     
 const ClassInfo JSString::s_info = { "string"_s, nullptr, nullptr, nullptr, CREATE_METHOD_TABLE(JSString) };
 
 Structure* JSString::createStructure(VM& vm, JSGlobalObject* globalObject, JSValue proto)
 {
-    return Structure::create(vm, globalObject, proto, TypeInfo(StringType, StructureFlags), info());
+    return Structure::create(vm, globalObject, proto, defaultTypeInfo(), info());
 }
 
 JSString* JSString::createEmptyString(VM& vm)
@@ -74,11 +76,11 @@ void JSString::dumpToStream(const JSCell* cell, PrintStream& out)
         else
             out.printf("[rope]");
     } else {
-        if (WTF::StringImpl* ourImpl = bitwise_cast<StringImpl*>(pointer)) {
+        if (WTF::StringImpl* ourImpl = std::bit_cast<StringImpl*>(pointer)) {
             if (ourImpl->is8Bit())
-                out.printf("[8 %p]", ourImpl->characters8());
+                out.printf("[8 %p]", ourImpl->span8().data());
             else
-                out.printf("[16 %p]", ourImpl->characters16());
+                out.printf("[16 %p]", ourImpl->span16().data());
         }
     }
     out.printf(">");
@@ -86,17 +88,7 @@ void JSString::dumpToStream(const JSCell* cell, PrintStream& out)
 
 bool JSString::equalSlowCase(JSGlobalObject* globalObject, JSString* other) const
 {
-    VM& vm = globalObject->vm();
-    auto scope = DECLARE_THROW_SCOPE(vm);
-
-    if (length() != other->length())
-        return false;
-
-    String str1 = value(globalObject);
-    RETURN_IF_EXCEPTION(scope, false);
-    String str2 = other->value(globalObject);
-    RETURN_IF_EXCEPTION(scope, false);
-    return WTF::equal(*str1.impl(), *str2.impl());
+    return equalInline(globalObject, other);
 }
 
 size_t JSString::estimatedSize(JSCell* cell, VM& vm)
@@ -105,7 +97,7 @@ size_t JSString::estimatedSize(JSCell* cell, VM& vm)
     uintptr_t pointer = thisObject->fiberConcurrently();
     if (pointer & isRopeInPointer)
         return Base::estimatedSize(cell, vm);
-    return Base::estimatedSize(cell, vm) + bitwise_cast<StringImpl*>(pointer)->costDuringGC();
+    return Base::estimatedSize(cell, vm) + std::bit_cast<StringImpl*>(pointer)->costDuringGC();
 }
 
 template<typename Visitor>
@@ -125,7 +117,7 @@ void JSString::visitChildrenImpl(JSCell* cell, Visitor& visitor)
             JSString* fiber = nullptr;
             switch (index) {
             case 0:
-                fiber = bitwise_cast<JSString*>(pointer & JSRopeString::stringMask);
+                fiber = std::bit_cast<JSString*>(pointer & JSRopeString::stringMask);
                 break;
             case 1:
                 fiber = static_cast<JSRopeString*>(thisObject)->fiber1();
@@ -143,136 +135,97 @@ void JSString::visitChildrenImpl(JSCell* cell, Visitor& visitor)
         }
         return;
     }
-    if (StringImpl* impl = bitwise_cast<StringImpl*>(pointer))
+    if (StringImpl* impl = std::bit_cast<StringImpl*>(pointer))
         visitor.reportExtraMemoryVisited(impl->costDuringGC());
 }
 
 DEFINE_VISIT_CHILDREN(JSString);
 
-static constexpr unsigned maxLengthForOnStackResolve = 2048;
-
 template<typename CharacterType>
-void JSRopeString::resolveRopeInternal(CharacterType* buffer) const
+void JSRopeString::resolveRopeInternalNoSubstring(std::span<CharacterType> buffer, uint8_t* stackLimit) const
 {
-    if (isSubstring()) {
-        // It is possible that underlying string becomes 8bit/16bit while wrapper substring is saying it is 16bit/8bit.
-        // But It is definitely true that substring part can be represented as its parent's status 8bit/16bit, which is described as CharacterType.
-        auto& string = substringBase()->valueInternal();
-        if (string.is8Bit())
-            StringImpl::copyCharacters(buffer, string.characters8() + substringOffset(), length());
-        else
-            StringImpl::copyCharacters(buffer, string.characters16() + substringOffset(), length());
-        return;
-    }
-    
-    resolveRopeInternalNoSubstring(buffer);
+    resolveToBuffer(fiber0(), fiber1(), fiber2(), buffer, stackLimit);
 }
 
-template<typename CharacterType>
-void JSRopeString::resolveRopeInternalNoSubstring(CharacterType* buffer) const
-{
-    for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i) {
-        if (fiber(i)->isRope()) {
-            resolveRopeSlowCase(buffer);
-            return;
-        }
-    }
-
-    CharacterType* position = buffer;
-    for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i) {
-        const StringImpl& fiberString = *fiber(i)->valueInternal().impl();
-        unsigned length = fiberString.length();
-        if (fiberString.is8Bit())
-            StringImpl::copyCharacters(position, fiberString.characters8(), length);
-        else
-            StringImpl::copyCharacters(position, fiberString.characters16(), length);
-        position += length;
-    }
-    ASSERT((buffer + length()) == position);
-}
-
-AtomString JSRopeString::resolveRopeToAtomString(JSGlobalObject* globalObject) const
+GCOwnedDataScope<AtomStringImpl*> JSRopeString::resolveRopeToAtomString(JSGlobalObject* globalObject) const
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-    auto convertToAtomString = [](const String& string) -> AtomString {
+    auto convertToAtomString = [this](const String& string) -> GCOwnedDataScope<AtomStringImpl*> {
         ASSERT(!string.impl() || string.impl()->isAtom());
-        return static_cast<AtomStringImpl*>(string.impl());
+        return { this, static_cast<AtomStringImpl*>(string.impl()) };
     };
 
     if (length() > maxLengthForOnStackResolve) {
         scope.release();
-        return convertToAtomString(resolveRopeWithFunction(globalObject, [&] (Ref<StringImpl>&& newImpl) {
+        constexpr bool reportAllocation = true;
+        return convertToAtomString(resolveRopeWithFunction<reportAllocation>(globalObject, [&] (Ref<StringImpl>&& newImpl) {
             return AtomStringImpl::add(newImpl.ptr());
         }));
     }
 
-    if (is8Bit()) {
-        LChar buffer[maxLengthForOnStackResolve];
-        resolveRopeInternal(buffer);
-        convertToNonRope(AtomStringImpl::add(buffer, length()));
-    } else {
-        UChar buffer[maxLengthForOnStackResolve];
-        resolveRopeInternal(buffer);
-        convertToNonRope(AtomStringImpl::add(buffer, length()));
-    }
+    AtomString atomString;
+    uint8_t* stackLimit = std::bit_cast<uint8_t*>(vm.softStackLimit());
+    if (!isSubstring()) {
+        if (is8Bit()) {
+            std::array<LChar, maxLengthForOnStackResolve> buffer;
+            resolveRopeInternalNoSubstring(std::span { buffer }.first(length()), stackLimit);
+            atomString = std::span<const LChar> { buffer }.first(length());
+        } else {
+            std::array<char16_t, maxLengthForOnStackResolve> buffer;
+            resolveRopeInternalNoSubstring(std::span { buffer }.first(length()), stackLimit);
+            atomString = std::span<const char16_t> { buffer }.first(length());
+        }
+    } else
+        atomString = StringView { substringBase()->valueInternal() }.substring(substringOffset(), length()).toAtomString();
 
+    size_t sizeToReport = atomString.impl()->hasOneRef() ? atomString.impl()->cost() : 0;
+    convertToNonRope(String { atomString.releaseImpl() });
     // If we resolved a string that didn't previously exist, notify the heap that we've grown.
-    if (valueInternal().impl()->hasOneRef())
-        vm.heap.reportExtraMemoryAllocated(valueInternal().impl()->cost());
-    return convertToAtomString(valueInternal());
+    vm.heap.reportExtraMemoryAllocated(this, sizeToReport);
+    return { this, static_cast<AtomStringImpl*>(valueInternal().impl()) };
 }
 
-inline void JSRopeString::convertToNonRope(String&& string) const
-{
-    // Concurrent compiler threads can access String held by JSString. So we always emit
-    // store-store barrier here to ensure concurrent compiler threads see initialized String.
-    ASSERT(JSString::isRope());
-    WTF::storeStoreFence();
-    new (&uninitializedValueInternal()) String(WTFMove(string));
-    static_assert(sizeof(String) == sizeof(RefPtr<StringImpl>), "JSString's String initialization must be done in one pointer move.");
-    // We do not clear the trailing fibers and length information (fiber1 and fiber2) because we could be reading the length concurrently.
-    ASSERT(!JSString::isRope());
-}
-
-RefPtr<AtomStringImpl> JSRopeString::resolveRopeToExistingAtomString(JSGlobalObject* globalObject) const
+GCOwnedDataScope<AtomStringImpl*> JSRopeString::resolveRopeToExistingAtomString(JSGlobalObject* globalObject) const
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (length() > maxLengthForOnStackResolve) {
         RefPtr<AtomStringImpl> existingAtomString;
-        resolveRopeWithFunction(globalObject, [&] (Ref<StringImpl>&& newImpl) -> Ref<StringImpl> {
+        constexpr bool reportAllocation = true;
+        resolveRopeWithFunction<reportAllocation>(globalObject, [&] (Ref<StringImpl>&& newImpl) -> Ref<StringImpl> {
             existingAtomString = AtomStringImpl::lookUp(newImpl.ptr());
             if (existingAtomString)
                 return Ref { *existingAtomString };
             return WTFMove(newImpl);
         });
-        RETURN_IF_EXCEPTION(scope, nullptr);
-        return existingAtomString;
+        RETURN_IF_EXCEPTION(scope, { });
+        return { this, existingAtomString.get() };
     }
     
-    if (is8Bit()) {
-        LChar buffer[maxLengthForOnStackResolve];
-        resolveRopeInternal(buffer);
-        if (RefPtr<AtomStringImpl> existingAtomString = AtomStringImpl::lookUp(buffer, length())) {
-            convertToNonRope(*existingAtomString);
-            return existingAtomString;
+    RefPtr<AtomStringImpl> existingAtomString;
+    if (!isSubstring()) {
+        uint8_t* stackLimit = std::bit_cast<uint8_t*>(vm.softStackLimit());
+        if (is8Bit()) {
+            std::array<LChar, maxLengthForOnStackResolve> buffer;
+            resolveRopeInternalNoSubstring(std::span { buffer }.first(length()), stackLimit);
+            existingAtomString = AtomStringImpl::lookUp(std::span { buffer }.first(length()));
+        } else {
+            std::array<char16_t, maxLengthForOnStackResolve> buffer;
+            resolveRopeInternalNoSubstring(std::span { buffer }.first(length()), stackLimit);
+            existingAtomString = AtomStringImpl::lookUp(std::span { buffer }.first(length()));
         }
-    } else {
-        UChar buffer[maxLengthForOnStackResolve];
-        resolveRopeInternal(buffer);
-        if (RefPtr<AtomStringImpl> existingAtomString = AtomStringImpl::lookUp(buffer, length())) {
-            convertToNonRope(*existingAtomString);
-            return existingAtomString;
-        }
-    }
+    } else
+        existingAtomString = StringView { substringBase()->valueInternal() }.substring(substringOffset(), length()).toExistingAtomString().releaseImpl();
 
-    return nullptr;
+    if (existingAtomString)
+        convertToNonRope(*existingAtomString);
+    return { this, existingAtomString.get() };
 }
 
-template<typename Function>
+template<bool reportAllocation, typename Function>
 const String& JSRopeString::resolveRopeWithFunction(JSGlobalObject* nullOrGlobalObjectForOOM, Function&& function) const
 {
     ASSERT(isRope());
@@ -286,92 +239,52 @@ const String& JSRopeString::resolveRopeWithFunction(JSGlobalObject* nullOrGlobal
     }
     
     if (is8Bit()) {
-        LChar* buffer;
+        std::span<LChar> buffer;
         auto newImpl = StringImpl::tryCreateUninitialized(length(), buffer);
         if (!newImpl) {
             outOfMemory(nullOrGlobalObjectForOOM);
             return nullString();
         }
-        vm.heap.reportExtraMemoryAllocated(newImpl->cost());
 
-        resolveRopeInternalNoSubstring(buffer);
+        size_t sizeToReport = newImpl->cost();
+        uint8_t* stackLimit = std::bit_cast<uint8_t*>(vm.softStackLimit());
+        resolveRopeInternalNoSubstring(buffer, stackLimit);
         convertToNonRope(function(newImpl.releaseNonNull()));
+        if constexpr (reportAllocation)
+            vm.heap.reportExtraMemoryAllocated(this, sizeToReport);
         return valueInternal();
     }
     
-    UChar* buffer;
+    std::span<char16_t> buffer;
     auto newImpl = StringImpl::tryCreateUninitialized(length(), buffer);
     if (!newImpl) {
         outOfMemory(nullOrGlobalObjectForOOM);
         return nullString();
     }
-    vm.heap.reportExtraMemoryAllocated(newImpl->cost());
     
-    resolveRopeInternalNoSubstring(buffer);
+    size_t sizeToReport = newImpl->cost();
+    uint8_t* stackLimit = std::bit_cast<uint8_t*>(vm.softStackLimit());
+    resolveRopeInternalNoSubstring(buffer, stackLimit);
     convertToNonRope(function(newImpl.releaseNonNull()));
+    if constexpr (reportAllocation)
+        vm.heap.reportExtraMemoryAllocated(this, sizeToReport);
     return valueInternal();
 }
 
 const String& JSRopeString::resolveRope(JSGlobalObject* nullOrGlobalObjectForOOM) const
 {
-    return resolveRopeWithFunction(nullOrGlobalObjectForOOM, [] (Ref<StringImpl>&& newImpl) {
+    constexpr bool reportAllocation = true;
+    return resolveRopeWithFunction<reportAllocation>(nullOrGlobalObjectForOOM, [] (Ref<StringImpl>&& newImpl) {
         return WTFMove(newImpl);
     });
 }
 
-// Overview: These functions convert a JSString from holding a string in rope form
-// down to a simple String representation. It does so by building up the string
-// backwards, since we want to avoid recursion, we expect that the tree structure
-// representing the rope is likely imbalanced with more nodes down the left side
-// (since appending to the string is likely more common) - and as such resolving
-// in this fashion should minimize work queue size.  (If we built the queue forwards
-// we would likely have to place all of the constituent StringImpls into the
-// Vector before performing any concatenation, but by working backwards we likely
-// only fill the queue with the number of substrings at any given level in a
-// rope-of-ropes.)
-template<typename CharacterType>
-void JSRopeString::resolveRopeSlowCase(CharacterType* buffer) const
+const String& JSRopeString::resolveRopeWithoutGC() const
 {
-    CharacterType* position = buffer + length(); // We will be working backwards over the rope.
-    Vector<JSString*, 32, UnsafeVectorOverflow> workQueue; // These strings are kept alive by the parent rope, so using a Vector is OK.
-
-    for (size_t i = 0; i < s_maxInternalRopeLength && fiber(i); ++i)
-        workQueue.append(fiber(i));
-
-    while (!workQueue.isEmpty()) {
-        JSString* currentFiber = workQueue.last();
-        workQueue.removeLast();
-
-        if (currentFiber->isRope()) {
-            JSRopeString* currentFiberAsRope = static_cast<JSRopeString*>(currentFiber);
-            if (currentFiberAsRope->isSubstring()) {
-                ASSERT(!currentFiberAsRope->substringBase()->isRope());
-                StringImpl* string = static_cast<StringImpl*>(
-                    currentFiberAsRope->substringBase()->valueInternal().impl());
-                unsigned offset = currentFiberAsRope->substringOffset();
-                unsigned length = currentFiberAsRope->length();
-                position -= length;
-                if (string->is8Bit())
-                    StringImpl::copyCharacters(position, string->characters8() + offset, length);
-                else
-                    StringImpl::copyCharacters(position, string->characters16() + offset, length);
-                continue;
-            }
-            for (size_t i = 0; i < s_maxInternalRopeLength && currentFiberAsRope->fiber(i); ++i)
-                workQueue.append(currentFiberAsRope->fiber(i));
-            continue;
-        }
-
-        StringImpl* string = static_cast<StringImpl*>(currentFiber->valueInternal().impl());
-        unsigned length = string->length();
-        position -= length;
-        if (string->is8Bit())
-            StringImpl::copyCharacters(position, string->characters8(), length);
-        else
-            StringImpl::copyCharacters(position, string->characters16(), length);
-    }
-
-    ASSERT(buffer == position);
+    constexpr bool reportAllocation = false;
+    return resolveRopeWithFunction<reportAllocation>(nullptr, [] (Ref<StringImpl>&& newImpl) {
+        return WTFMove(newImpl);
+    });
 }
 
 void JSRopeString::outOfMemory(JSGlobalObject* nullOrGlobalObjectForOOM) const
@@ -393,7 +306,7 @@ double JSString::toNumber(JSGlobalObject* globalObject) const
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
-    StringView view = unsafeView(globalObject);
+    auto view = this->view(globalObject);
     RETURN_IF_EXCEPTION(scope, 0);
     return jsToNumber(view);
 }
@@ -410,13 +323,6 @@ JSObject* JSString::toObject(JSGlobalObject* globalObject) const
     return StringObject::create(globalObject->vm(), globalObject, const_cast<JSString*>(this));
 }
 
-JSValue JSString::toThis(JSCell* cell, JSGlobalObject* globalObject, ECMAMode ecmaMode)
-{
-    if (ecmaMode.isStrict())
-        return cell;
-    return StringObject::create(globalObject->vm(), globalObject, asString(cell));
-}
-
 bool JSString::getStringPropertyDescriptor(JSGlobalObject* globalObject, PropertyName propertyName, PropertyDescriptor& descriptor)
 {
     VM& vm = globalObject->vm();
@@ -434,11 +340,23 @@ bool JSString::getStringPropertyDescriptor(JSGlobalObject* globalObject, Propert
     return false;
 }
 
+GCOwnedDataScope<const String&> JSString::tryGetValueWithoutGC() const
+{
+    if (isRope()) {
+        // Pass nullptr for the JSGlobalObject so that resolveRope does not throw in the event of an OOM error.
+        return { this, static_cast<const JSRopeString*>(this)->resolveRopeWithoutGC() };
+    }
+    return { this, valueInternal() };
+}
+
 JSString* jsStringWithCacheSlowCase(VM& vm, StringImpl& stringImpl)
 {
-    JSString* string = jsString(vm, String(stringImpl));
-    vm.lastCachedString.set(vm, string);
+    ASSERT(stringImpl.length() > 1 || (stringImpl.length() == 1 && stringImpl[0] > maxSingleCharacterString));
+    JSString* string = JSString::create(vm, stringImpl);
+    vm.lastCachedString.setWithoutWriteBarrier(string);
     return string;
 }
 
 } // namespace JSC
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
