@@ -33,7 +33,7 @@ class ProgramPipeline;
 class ProgramPipelineState final : angle::NonCopyable
 {
   public:
-    ProgramPipelineState();
+    ProgramPipelineState(rx::GLImplFactory *factory);
     ~ProgramPipelineState();
 
     const std::string &getLabel() const;
@@ -44,29 +44,45 @@ class ProgramPipelineState final : angle::NonCopyable
         return *mExecutable;
     }
 
+    const SharedProgramExecutable &getSharedExecutable() const
+    {
+        ASSERT(mExecutable);
+        return mExecutable;
+    }
+
     void activeShaderProgram(Program *shaderProgram);
     void useProgramStages(const Context *context,
-                          GLbitfield stages,
+                          const gl::ShaderBitSet &shaderTypes,
                           Program *shaderProgram,
-                          std::vector<angle::ObserverBinding> *programObserverBindings);
+                          std::vector<angle::ObserverBinding> *programObserverBindings,
+                          std::vector<angle::ObserverBinding> *programExecutableObserverBindings);
 
     Program *getActiveShaderProgram() { return mActiveShaderProgram; }
 
     GLboolean isValid() const { return mValid; }
 
     const Program *getShaderProgram(ShaderType shaderType) const { return mPrograms[shaderType]; }
+    const SharedProgramExecutable &getShaderProgramExecutable(ShaderType shaderType) const
+    {
+        return mExecutable->mPPOProgramExecutables[shaderType];
+    }
 
     bool usesShaderProgram(ShaderProgramID program) const;
 
     void updateExecutableTextures();
 
-    rx::SpecConstUsageBits getSpecConstUsageBits() const;
+    void updateExecutableSpecConstUsageBits();
 
   private:
+    SharedProgramExecutable makeNewExecutable(
+        rx::GLImplFactory *factory,
+        ShaderMap<SharedProgramExecutable> &&ppoProgramExecutables);
     void useProgramStage(const Context *context,
                          ShaderType shaderType,
                          Program *shaderProgram,
-                         angle::ObserverBinding *programObserverBindings);
+                         angle::ObserverBinding *programObserverBinding,
+                         angle::ObserverBinding *programExecutableObserverBinding);
+    void destroyDiscardedExecutables(const Context *context);
 
     friend class ProgramPipeline;
 
@@ -77,9 +93,19 @@ class ProgramPipelineState final : angle::NonCopyable
     // The shader programs for each stage.
     ShaderMap<Program *> mPrograms;
 
+    // Mapping from program's UBOs into the program executable's UBOs.
+    ShaderMap<ProgramUniformBlockArray<GLuint>> mUniformBlockMap;
+
+    // A list of executables to be garbage collected.  This is populated as the pipeline is
+    // notified about program relinks, but cannot immediately destroy the old executables due to
+    // lack of access to context.
+    std::vector<SharedProgramExecutable> mProgramExecutablesToDiscard;
+
     GLboolean mValid;
 
-    ProgramExecutable *mExecutable;
+    InfoLog mInfoLog;
+
+    SharedProgramExecutable mExecutable;
 
     bool mIsLinked;
 };
@@ -95,13 +121,17 @@ class ProgramPipeline final : public RefCountObject<ProgramPipelineID>,
 
     void onDestroy(const Context *context) override;
 
-    void setLabel(const Context *context, const std::string &label) override;
+    angle::Result setLabel(const Context *context, const std::string &label) override;
     const std::string &getLabel() const override;
 
     const ProgramPipelineState &getState() const { return mState; }
     ProgramPipelineState &getState() { return mState; }
 
     ProgramExecutable &getExecutable() const { return mState.getExecutable(); }
+    const SharedProgramExecutable &getSharedExecutable() const
+    {
+        return mState.getSharedExecutable();
+    }
 
     rx::ProgramPipelineImpl *getImplementation() const;
 
@@ -121,11 +151,42 @@ class ProgramPipeline final : public RefCountObject<ProgramPipelineID>,
                                    GLbitfield stages,
                                    Program *shaderProgram);
 
-    Program *getShaderProgram(ShaderType shaderType) const { return mState.mPrograms[shaderType]; }
+    const Program *getShaderProgram(ShaderType shaderType) const
+    {
+        return mState.getShaderProgram(shaderType);
+    }
+    const SharedProgramExecutable &getShaderProgramExecutable(ShaderType shaderType) const
+    {
+        return mState.getShaderProgramExecutable(shaderType);
+    }
 
     void resetIsLinked() { mState.mIsLinked = false; }
     angle::Result link(const gl::Context *context);
-    bool linkVaryings(InfoLog &infoLog) const;
+
+    InfoLog &getInfoLog() { return mState.mInfoLog; }
+    int getInfoLogLength() const;
+    void getInfoLog(GLsizei bufSize, GLsizei *length, char *infoLog) const;
+
+    // Ensure program pipeline is linked. Inlined to make sure its overhead is as low as possible.
+    void resolveLink(const Context *context)
+    {
+        if (mState.mIsLinked)
+        {
+            // Already linked, nothing to do.
+            return;
+        }
+
+        resolveAttachedPrograms(context);
+        angle::Result linkResult = link(context);
+        if (linkResult != angle::Result::Continue)
+        {
+            // If the link failed then log a warning, swallow the error and move on.
+            WARN() << "ProgramPipeline link failed" << std::endl;
+        }
+        return;
+    }
+    void resolveAttachedPrograms(const Context *context);
+
     void validate(const gl::Context *context);
     GLboolean isValid() const { return mState.isValid(); }
     bool isLinked() const { return mState.mIsLinked; }
@@ -134,6 +195,7 @@ class ProgramPipeline final : public RefCountObject<ProgramPipelineID>,
     void onSubjectStateChange(angle::SubjectIndex index, angle::SubjectMessage message) override;
 
   private:
+    bool linkVaryings();
     void updateLinkedShaderStages();
     void updateExecutableAttributes();
     void updateTransformFeedbackMembers();
@@ -150,7 +212,7 @@ class ProgramPipeline final : public RefCountObject<ProgramPipelineID>,
     ProgramPipelineState mState;
 
     std::vector<angle::ObserverBinding> mProgramObserverBindings;
-    angle::ObserverBinding mExecutableObserverBinding;
+    std::vector<angle::ObserverBinding> mProgramExecutableObserverBindings;
 };
 }  // namespace gl
 
