@@ -12,7 +12,9 @@
 
 #include <string.h>
 
+#include "common/hash_containers.h"
 #include "common/system_utils.h"
+#include "platform/Feature.h"
 #include "platform/PlatformMethods.h"
 #include "util/OSWindow.h"
 
@@ -29,15 +31,29 @@ ConfigParameters::ConfigParameters()
       alphaBits(-1),
       depthBits(-1),
       stencilBits(-1),
+      // The default value of EGL_CONTEXT_WEBGL_COMPATIBILITY_ANGLE is EGL_FALSE.
+      webGLCompatibility(false),
+      // The default value of EGL_ROBUST_RESOURCE_INITIALIZATION_ANGLE is EGL_FALSE.
+      robustResourceInit(false),
       componentType(EGL_COLOR_COMPONENT_TYPE_FIXED_EXT),
       multisample(false),
+      // The default value of EGL_CONTEXT_OPENGL_DEBUG is EGL_FALSE.
       debug(false),
+      // The default value of EGL_CONTEXT_OPENGL_NO_ERROR_KHR is EGL_FALSE.
       noError(false),
+      // The default value of EGL_EXTENSIONS_ENABLED_ANGLE is EGL_TRUE.
+      extensionsEnabled(true),
+      // The default value of EGL_CONTEXT_BIND_GENERATES_RESOURCE_CHROMIUM is EGL_TRUE.
       bindGeneratesResource(true),
+      // The default value of CLIENT_ARRAYS_ANGLE is EGL_TRUE.
       clientArraysEnabled(true),
+      // The default value of EGL_CONTEXT_OPENGL_ROBUST_ACCESS_EXT is EGL_FALSE.
       robustAccess(false),
+      // EGL_RENDER_BUFFER requires EGL 1.4+ or extension support.
       mutableRenderBuffer(false),
       samples(-1),
+      // The default value of EGL_CONTEXT_PROGRAM_BINARY_CACHE_ENABLED_ANGLE is EGL_TRUE.
+      contextProgramCacheEnabled(true),
       resetStrategy(EGL_NO_RESET_NOTIFICATION_EXT),
       colorSpace(EGL_COLORSPACE_LINEAR),
       swapInterval(kDefaultSwapInterval)
@@ -51,7 +67,7 @@ void ConfigParameters::reset()
 }
 
 // GLWindowBase implementation.
-GLWindowBase::GLWindowBase(EGLint glesMajorVersion, EGLint glesMinorVersion)
+GLWindowBase::GLWindowBase(GLint glesMajorVersion, EGLint glesMinorVersion)
     : mClientMajorVersion(glesMajorVersion), mClientMinorVersion(glesMinorVersion)
 {}
 
@@ -66,7 +82,9 @@ EGLWindow::EGLWindow(EGLint glesMajorVersion, EGLint glesMinorVersion)
       mContext(EGL_NO_CONTEXT),
       mEGLMajorVersion(0),
       mEGLMinorVersion(0)
-{}
+{
+    std::fill(mFeatures.begin(), mFeatures.end(), ANGLEFeatureStatus::Unknown);
+}
 
 EGLWindow::~EGLWindow()
 {
@@ -140,6 +158,19 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
                                   angle::GLESDriverType driverType,
                                   const EGLPlatformParameters &params)
 {
+    if (driverType == angle::GLESDriverType::ZinkEGL)
+    {
+        std::stringstream driDirStream;
+        char s = angle::GetPathSeparator();
+        driDirStream << angle::GetModuleDirectory() << "mesa" << s << "src" << s << "gallium" << s
+                     << "targets" << s << "dri";
+
+        std::string driDir = driDirStream.str();
+
+        angle::SetEnvironmentVar("MESA_LOADER_DRIVER_OVERRIDE", "zink");
+        angle::SetEnvironmentVar("LIBGL_DRIVERS_PATH", driDir.c_str());
+    }
+
 #if defined(ANGLE_USE_UTIL_LOADER)
     PFNEGLGETPROCADDRESSPROC getProcAddress;
     glWindowingLibrary->getAs("eglGetProcAddress", &getProcAddress);
@@ -150,11 +181,17 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
     }
 
     // Likely we will need to use a fallback to Library::getAs on non-ANGLE platforms.
-    angle::LoadEGL(getProcAddress);
+    LoadUtilEGL(getProcAddress);
 #endif  // defined(ANGLE_USE_UTIL_LOADER)
 
+    // EGL_NO_DISPLAY + EGL_EXTENSIONS returns NULL before Android 10
     const char *extensionString =
         static_cast<const char *>(eglQueryString(EGL_NO_DISPLAY, EGL_EXTENSIONS));
+    if (!extensionString)
+    {
+        // fallback to an empty string for strstr
+        extensionString = "";
+    }
 
     std::vector<EGLAttrib> displayAttributes;
     displayAttributes.push_back(EGL_PLATFORM_ANGLE_TYPE_ANGLE);
@@ -249,8 +286,7 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
     if (driverType == angle::GLESDriverType::SystemWGL)
         return false;
 
-    if (driverType == angle::GLESDriverType::AngleEGL &&
-        strstr(extensionString, "EGL_ANGLE_platform_angle"))
+    if (IsANGLE(driverType) && strstr(extensionString, "EGL_ANGLE_platform_angle"))
     {
         mDisplay = eglGetPlatformDisplay(EGL_PLATFORM_ANGLE_ANGLE,
                                          reinterpret_cast<void *>(osWindow->getNativeDisplay()),
@@ -275,6 +311,8 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
         return false;
     }
 
+    queryFeatures();
+
     mPlatform = params;
     return true;
 }
@@ -357,11 +395,10 @@ GLWindowResult EGLWindow::initializeSurface(OSWindow *osWindow,
 
     bool hasRobustResourceInit =
         strstr(displayExtensions, "EGL_ANGLE_robust_resource_initialization") != nullptr;
-    if (hasRobustResourceInit && mConfigParams.robustResourceInit.valid())
+    if (hasRobustResourceInit)
     {
         surfaceAttributes.push_back(EGL_ROBUST_RESOURCE_INITIALIZATION_ANGLE);
-        surfaceAttributes.push_back(mConfigParams.robustResourceInit.value() ? EGL_TRUE
-                                                                             : EGL_FALSE);
+        surfaceAttributes.push_back(mConfigParams.robustResourceInit ? EGL_TRUE : EGL_FALSE);
     }
 
     bool hasGLColorSpace = strstr(displayExtensions, "EGL_KHR_gl_colorspace") != nullptr;
@@ -391,15 +428,16 @@ GLWindowResult EGLWindow::initializeSurface(OSWindow *osWindow,
 
     mSurface = eglCreateWindowSurface(mDisplay, mConfig, osWindow->getNativeWindow(),
                                       &surfaceAttributes[0]);
-    if (eglGetError() != EGL_SUCCESS || (mSurface == EGL_NO_SURFACE))
+    EGLint error = eglGetError();
+    if (error != EGL_SUCCESS || (mSurface == EGL_NO_SURFACE))
     {
-        fprintf(stderr, "eglCreateWindowSurface failed: 0x%X\n", eglGetError());
+        fprintf(stderr, "eglCreateWindowSurface failed: 0x%X\n", error);
         destroyGL();
         return GLWindowResult::Error;
     }
 
 #if defined(ANGLE_USE_UTIL_LOADER)
-    angle::LoadGLES(eglGetProcAddress);
+    LoadUtilGLES(eglGetProcAddress);
 #endif  // defined(ANGLE_USE_UTIL_LOADER)
 
     return GLWindowResult::NoError;
@@ -439,7 +477,7 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
 
     bool hasWebGLCompatibility =
         strstr(displayExtensions, "EGL_ANGLE_create_context_webgl_compatibility") != nullptr;
-    if (mConfigParams.webGLCompatibility.valid() && !hasWebGLCompatibility)
+    if (mConfigParams.webGLCompatibility && !hasWebGLCompatibility)
     {
         fprintf(stderr, "EGL_ANGLE_create_context_webgl_compatibility missing.\n");
         return EGL_NO_CONTEXT;
@@ -447,7 +485,7 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
 
     bool hasCreateContextExtensionsEnabled =
         strstr(displayExtensions, "EGL_ANGLE_create_context_extensions_enabled") != nullptr;
-    if (mConfigParams.extensionsEnabled.valid() && !hasCreateContextExtensionsEnabled)
+    if (!mConfigParams.extensionsEnabled && !hasCreateContextExtensionsEnabled)
     {
         fprintf(stderr, "EGL_ANGLE_create_context_extensions_enabled missing.\n");
         return EGL_NO_CONTEXT;
@@ -466,6 +504,7 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
         strstr(displayExtensions, "EGL_CHROMIUM_create_context_bind_generates_resource") != nullptr;
     if (!mConfigParams.bindGeneratesResource && !hasBindGeneratesResource)
     {
+        // Non-default state requested without the extension present
         fprintf(stderr, "EGL_CHROMIUM_create_context_bind_generates_resource missing.\n");
         return EGL_NO_CONTEXT;
     }
@@ -481,8 +520,9 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
 
     bool hasProgramCacheControlExtension =
         strstr(displayExtensions, "EGL_ANGLE_program_cache_control ") != nullptr;
-    if (mConfigParams.contextProgramCacheEnabled.valid() && !hasProgramCacheControlExtension)
+    if (!mConfigParams.contextProgramCacheEnabled && !hasProgramCacheControlExtension)
     {
+        // Non-default state requested without the extension present
         fprintf(stderr, "EGL_ANGLE_program_cache_control missing.\n");
         return EGL_NO_CONTEXT;
     }
@@ -495,6 +535,14 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
         return EGL_NO_CONTEXT;
     }
 
+    bool hasRobustResourceInit =
+        strstr(displayExtensions, "EGL_ANGLE_robust_resource_initialization") != nullptr;
+    if (mConfigParams.robustResourceInit && !hasRobustResourceInit)
+    {
+        fprintf(stderr, "EGL_ANGLE_robust_resource_initialization missing.\n");
+        return EGL_NO_CONTEXT;
+    }
+
     eglBindAPI(EGL_OPENGL_ES_API);
     if (eglGetError() != EGL_SUCCESS)
     {
@@ -526,7 +574,7 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
             contextAttributes.push_back(mConfigParams.debug ? EGL_TRUE : EGL_FALSE);
         }
 
-        // TODO (http://anglebug.com/5809)
+        // TODO (http://anglebug.com/42264345)
         // Mesa does not allow EGL_CONTEXT_OPENGL_NO_ERROR_KHR for GLES1.
         if (hasKHRCreateContextNoError && mConfigParams.noError)
         {
@@ -534,18 +582,16 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
             contextAttributes.push_back(mConfigParams.noError ? EGL_TRUE : EGL_FALSE);
         }
 
-        if (mConfigParams.webGLCompatibility.valid())
+        if (hasWebGLCompatibility)
         {
             contextAttributes.push_back(EGL_CONTEXT_WEBGL_COMPATIBILITY_ANGLE);
-            contextAttributes.push_back(mConfigParams.webGLCompatibility.value() ? EGL_TRUE
-                                                                                 : EGL_FALSE);
+            contextAttributes.push_back(mConfigParams.webGLCompatibility ? EGL_TRUE : EGL_FALSE);
         }
 
-        if (mConfigParams.extensionsEnabled.valid())
+        if (hasCreateContextExtensionsEnabled)
         {
             contextAttributes.push_back(EGL_EXTENSIONS_ENABLED_ANGLE);
-            contextAttributes.push_back(mConfigParams.extensionsEnabled.value() ? EGL_TRUE
-                                                                                : EGL_FALSE);
+            contextAttributes.push_back(mConfigParams.extensionsEnabled ? EGL_TRUE : EGL_FALSE);
         }
 
         if (hasRobustness)
@@ -569,11 +615,11 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
             contextAttributes.push_back(mConfigParams.clientArraysEnabled ? EGL_TRUE : EGL_FALSE);
         }
 
-        if (mConfigParams.contextProgramCacheEnabled.valid())
+        if (hasProgramCacheControlExtension)
         {
             contextAttributes.push_back(EGL_CONTEXT_PROGRAM_BINARY_CACHE_ENABLED_ANGLE);
-            contextAttributes.push_back(
-                mConfigParams.contextProgramCacheEnabled.value() ? EGL_TRUE : EGL_FALSE);
+            contextAttributes.push_back(mConfigParams.contextProgramCacheEnabled ? EGL_TRUE
+                                                                                 : EGL_FALSE);
         }
 
         bool hasBackwardsCompatibleContextExtension =
@@ -585,13 +631,10 @@ EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
             contextAttributes.push_back(EGL_FALSE);
         }
 
-        bool hasRobustResourceInit =
-            strstr(displayExtensions, "EGL_ANGLE_robust_resource_initialization") != nullptr;
-        if (hasRobustResourceInit && mConfigParams.robustResourceInit.valid())
+        if (hasRobustResourceInit)
         {
             contextAttributes.push_back(EGL_ROBUST_RESOURCE_INITIALIZATION_ANGLE);
-            contextAttributes.push_back(mConfigParams.robustResourceInit.value() ? EGL_TRUE
-                                                                                 : EGL_FALSE);
+            contextAttributes.push_back(mConfigParams.robustResourceInit ? EGL_TRUE : EGL_FALSE);
         }
     }
     contextAttributes.push_back(EGL_NONE);
@@ -621,6 +664,31 @@ bool EGLWindow::initializeContext()
         return false;
     }
 
+    // Without EGL_ANGLE_create_context_backwards_compatible and specifying
+    // EGL_CONTEXT_OPENGL_BACKWARDS_COMPATIBLE_ANGLE = EGL_FALSE, ANGLE will create a context with
+    // the maximum conformant version the display supports. If the extension is not supported, we
+    // need to query the actual context version, so each test can behave accordingly.
+    // EGL 1.5 Spec
+    // 3.7.1.1 OpenGL and OpenGL ES Context Versions
+    //   The context returned must be the specified version, or a later version which is
+    //   backwards compatible with that version.
+    bool hasBackwardsCompatibleContextExtension = angle::CheckExtensionExists(
+        eglQueryString(mDisplay, EGL_EXTENSIONS), "EGL_ANGLE_create_context_backwards_compatible");
+    if (!hasBackwardsCompatibleContextExtension)
+    {
+        std::pair<EGLint, EGLint> version = angle::GetCurrentContextVersion();
+        // There's no OpenGL ES version "0.x". There is "x.0", though, so we can only check the
+        // first.
+        if (version.first == 0)
+        {
+            fprintf(stderr, "Failed to query the current context version: 0x%X\n", eglGetError());
+            return false;
+        }
+
+        mClientMajorVersion = version.first;
+        mClientMinorVersion = version.second;
+    }
+
     return true;
 }
 
@@ -743,21 +811,103 @@ EGLBoolean EGLWindow::destroyImageKHR(Image image)
     return eglDestroyImageKHR(getDisplay(), image);
 }
 
-bool EGLWindow::makeCurrent(EGLContext context)
+EGLWindow::Sync EGLWindow::createSync(EGLDisplay dpy, EGLenum type, const EGLAttrib *attrib_list)
 {
+    return eglCreateSync(dpy, type, attrib_list);
+}
+
+EGLWindow::Sync EGLWindow::createSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list)
+{
+    return eglCreateSyncKHR(dpy, type, attrib_list);
+}
+
+EGLBoolean EGLWindow::destroySync(EGLDisplay dpy, Sync sync)
+{
+    return eglDestroySync(dpy, sync);
+}
+
+EGLBoolean EGLWindow::destroySyncKHR(EGLDisplay dpy, Sync sync)
+{
+    return eglDestroySyncKHR(dpy, sync);
+}
+
+EGLint EGLWindow::clientWaitSync(EGLDisplay dpy, Sync sync, EGLint flags, EGLTimeKHR timeout)
+{
+    return eglClientWaitSync(dpy, sync, flags, timeout);
+}
+
+EGLint EGLWindow::clientWaitSyncKHR(EGLDisplay dpy, Sync sync, EGLint flags, EGLTimeKHR timeout)
+{
+    return eglClientWaitSyncKHR(dpy, sync, flags, timeout);
+}
+
+EGLint EGLWindow::getEGLError()
+{
+    return eglGetError();
+}
+
+EGLWindow::Display EGLWindow::getCurrentDisplay()
+{
+    return eglGetCurrentDisplay();
+}
+
+GLWindowBase::Surface EGLWindow::createPbufferSurface(const EGLint *attrib_list)
+{
+    return eglCreatePbufferSurface(getDisplay(), getConfig(), attrib_list);
+}
+
+EGLBoolean EGLWindow::destroySurface(Surface surface)
+{
+    return eglDestroySurface(getDisplay(), surface);
+}
+
+EGLBoolean EGLWindow::bindTexImage(EGLSurface surface, EGLint buffer)
+{
+    return eglBindTexImage(getDisplay(), surface, buffer);
+}
+
+EGLBoolean EGLWindow::releaseTexImage(EGLSurface surface, EGLint buffer)
+{
+    return eglReleaseTexImage(getDisplay(), surface, buffer);
+}
+
+bool EGLWindow::makeCurrent(EGLSurface draw, EGLSurface read, EGLContext context)
+{
+    if ((draw && !read) || (!draw && read))
+    {
+        fprintf(stderr, "eglMakeCurrent: setting only one of draw and read buffer is illegal\n");
+        return false;
+    }
+
+    // if the draw buffer is a nullptr and a context is given, then we use mSurface,
+    // because we didn't add this the gSurfaceMap, and it is the most likely
+    // case that we actually wanted the default surface here.
+    // TODO: This will need additional work when we want to support capture/replay
+    // with a sourfaceless context.
+    //
+    // If no context is given then we also don't assign a surface
+    if (!draw)
+    {
+        draw = read = context != EGL_NO_CONTEXT ? mSurface : EGL_NO_SURFACE;
+    }
+
     if (isGLInitialized())
     {
-        if (eglMakeCurrent(mDisplay, mSurface, mSurface, context) == EGL_FALSE ||
+        if (eglMakeCurrent(mDisplay, draw, read, context) == EGL_FALSE ||
             eglGetError() != EGL_SUCCESS)
         {
             fprintf(stderr, "Error during eglMakeCurrent.\n");
             return false;
         }
     }
-
     return true;
 }
 
+bool EGLWindow::makeCurrent(EGLContext context)
+{
+    return makeCurrent(mSurface, mSurface, context);
+}
+
 bool EGLWindow::setSwapInterval(EGLint swapInterval)
 {
     if (eglSwapInterval(mDisplay, swapInterval) == EGL_FALSE || eglGetError() != EGL_SUCCESS)
@@ -798,3 +948,47 @@ void EGLWindow::Delete(EGLWindow **window)
     delete *window;
     *window = nullptr;
 }
+
+void EGLWindow::queryFeatures()
+{
+    // EGL_NO_DISPLAY + EGL_EXTENSIONS returns NULL before Android 10
+    const char *extensionString =
+        static_cast<const char *>(eglQueryString(EGL_NO_DISPLAY, EGL_EXTENSIONS));
+    const bool hasFeatureControlANGLE =
+        extensionString && strstr(extensionString, "EGL_ANGLE_feature_control") != nullptr;
+
+    if (!hasFeatureControlANGLE)
+    {
+        return;
+    }
+
+    angle::HashMap<std::string, angle::Feature> featureFromName;
+    for (angle::Feature feature : angle::AllEnums<angle::Feature>())
+    {
+        featureFromName[angle::GetFeatureName(feature)] = feature;
+    }
+
+    EGLAttrib featureCount = -1;
+    eglQueryDisplayAttribANGLE(mDisplay, EGL_FEATURE_COUNT_ANGLE, &featureCount);
+
+    for (int index = 0; index < featureCount; index++)
+    {
+        const char *featureName   = eglQueryStringiANGLE(mDisplay, EGL_FEATURE_NAME_ANGLE, index);
+        const char *featureStatus = eglQueryStringiANGLE(mDisplay, EGL_FEATURE_STATUS_ANGLE, index);
+        ASSERT(featureName != nullptr);
+        ASSERT(featureStatus != nullptr);
+
+        const angle::Feature feature = featureFromName[featureName];
+
+        const bool isEnabled  = strcmp(featureStatus, angle::kFeatureStatusEnabled) == 0;
+        const bool isDisabled = strcmp(featureStatus, angle::kFeatureStatusDisabled) == 0;
+        ASSERT(isEnabled || isDisabled);
+
+        mFeatures[feature] = isEnabled ? ANGLEFeatureStatus::Enabled : ANGLEFeatureStatus::Disabled;
+    }
+}
+
+bool EGLWindow::isFeatureEnabled(angle::Feature feature)
+{
+    return mFeatures[feature] == ANGLEFeatureStatus::Enabled;
+}
