@@ -27,23 +27,24 @@
 #include "FormDataConsumer.h"
 
 #include "BlobLoader.h"
+#include "ExceptionOr.h"
 #include "FormData.h"
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/WorkQueue.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(FormDataConsumer);
+
 FormDataConsumer::FormDataConsumer(const FormData& formData, ScriptExecutionContext& context, Callback&& callback)
     : m_formData(formData.copy())
     , m_context(&context)
     , m_callback(WTFMove(callback))
-    , m_fileQueue(WorkQueue::create("FormDataConsumer file queue"))
+    , m_fileQueue(WorkQueue::create("FormDataConsumer file queue"_s))
 {
-    read();
 }
 
-FormDataConsumer::~FormDataConsumer()
-{
-}
+FormDataConsumer::~FormDataConsumer() = default;
 
 void FormDataConsumer::read()
 {
@@ -54,7 +55,8 @@ void FormDataConsumer::read()
     ASSERT(!m_blobLoader);
 
     if (m_currentElementIndex >= m_formData->elements().size()) {
-        m_callback(Span<const uint8_t> { });
+        auto callback = std::exchange(m_callback, nullptr);
+        callback(std::span<const uint8_t> { });
         return;
     }
 
@@ -74,59 +76,70 @@ void FormDataConsumer::consumeData(const Vector<uint8_t>& content)
 
 void FormDataConsumer::consumeFile(const String& filename)
 {
+    m_isReadingFile = true;
     m_fileQueue->dispatch([weakThis = WeakPtr { *this }, identifier = m_context->identifier(), path = filename.isolatedCopy()]() mutable {
         ScriptExecutionContext::postTaskTo(identifier, [weakThis = WTFMove(weakThis), content = FileSystem::readEntireFile(path)](auto&) {
-            if (!weakThis)
+            RefPtr protectedThis = weakThis.get();
+            if (!protectedThis || !protectedThis->m_isReadingFile)
                 return;
 
+            protectedThis->m_isReadingFile = false;
             if (!content) {
-                weakThis->didFail(Exception { InvalidStateError, "Unable to read form data file"_s });
+                protectedThis->didFail(Exception { ExceptionCode::InvalidStateError, "Unable to read form data file"_s });
                 return;
             }
 
-            weakThis->consume(*content);
+            protectedThis->consume(*content);
         });
     });
 }
 
 void FormDataConsumer::consumeBlob(const URL& blobURL)
 {
-    m_blobLoader = makeUnique<BlobLoader>([weakThis = WeakPtr { *this }](BlobLoader&) mutable {
-        if (!weakThis)
+    m_blobLoader = BlobLoader::create([weakThis = WeakPtr { *this }](BlobLoader&) mutable {
+        RefPtr protectedThis = weakThis.get();
+        if (!protectedThis)
             return;
 
-        auto loader = std::exchange(weakThis->m_blobLoader, { });
+        auto loader = std::exchange(protectedThis->m_blobLoader, { });
         if (!loader)
             return;
 
         if (auto optionalErrorCode = loader->errorCode()) {
-            weakThis->didFail(Exception { InvalidStateError, "Failed to read form data blob"_s });
+            protectedThis->didFail(Exception { ExceptionCode::InvalidStateError, "Failed to read form data blob"_s });
             return;
         }
 
         if (auto data = loader->arrayBufferResult())
-            weakThis->consume(Span<const uint8_t> { static_cast<const uint8_t*>(data->data()), data->byteLength() });
+            protectedThis->consume(data->span());
     });
 
     m_blobLoader->start(blobURL, m_context.get(), FileReaderLoader::ReadAsArrayBuffer);
 
-    if (!m_blobLoader || !m_blobLoader->isLoading())
-        didFail(Exception { InvalidStateError, "Unable to read form data blob"_s });
+    if (RefPtr blobLoader = m_blobLoader.get())
+        didFail(Exception { ExceptionCode::InvalidStateError, "Unable to read form data blob"_s });
 }
 
-void FormDataConsumer::consume(Span<const uint8_t> content)
+void FormDataConsumer::consume(std::span<const uint8_t> content)
 {
     if (!m_callback)
         return;
 
-    m_callback(WTFMove(content));
-    if (!m_callback)
-        return;
+    if (!content.empty()) {
+        bool result = m_callback(WTFMove(content));
+        if (!result) {
+            cancel();
+            return;
+        }
+
+        if (!m_callback)
+            return;
+    }
 
     read();
 }
 
-void FormDataConsumer::didFail(auto&& exception)
+void FormDataConsumer::didFail(Exception&& exception)
 {
     auto callback = std::exchange(m_callback, nullptr);
     cancel();
@@ -139,6 +152,7 @@ void FormDataConsumer::cancel()
     m_callback = nullptr;
     if (auto loader = std::exchange(m_blobLoader, { }))
         loader->cancel();
+    m_isReadingFile = false;
     m_context = nullptr;
 }
 
