@@ -26,52 +26,62 @@
 #include "config.h"
 #include "IDBConnectionProxy.h"
 
-#if ENABLE(INDEXED_DATABASE)
-
 #include "IDBCursorInfo.h"
 #include "IDBDatabase.h"
+#include "IDBDatabaseNameAndVersion.h"
+#include "IDBDatabaseNameAndVersionRequest.h"
 #include "IDBGetRecordData.h"
 #include "IDBIterateCursorData.h"
 #include "IDBKeyRangeData.h"
 #include "IDBOpenDBRequest.h"
+#include "IDBOpenRequestData.h"
 #include "IDBRequestData.h"
 #include "IDBResultData.h"
 #include "ScriptExecutionContext.h"
 #include "SecurityOrigin.h"
 #include <wtf/MainThread.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 namespace IDBClient {
 
-IDBConnectionProxy::IDBConnectionProxy(IDBConnectionToServer& connection)
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(IDBConnectionProxy);
+
+IDBConnectionProxy::IDBConnectionProxy(IDBConnectionToServer& connection, PAL::SessionID sessionID)
     : m_connectionToServer(connection)
     , m_serverConnectionIdentifier(connection.identifier())
+    , m_sessionID(sessionID)
 {
     ASSERT(isMainThread());
 }
 
 void IDBConnectionProxy::ref()
 {
-    m_connectionToServer.ref();
+    m_connectionToServer->ref();
 }
 
 void IDBConnectionProxy::deref()
 {
-    m_connectionToServer.deref();
+    m_connectionToServer->deref();
+}
+
+PAL::SessionID IDBConnectionProxy::sessionID() const
+{
+    return m_sessionID;
 }
 
 Ref<IDBOpenDBRequest> IDBConnectionProxy::openDatabase(ScriptExecutionContext& context, const IDBDatabaseIdentifier& databaseIdentifier, uint64_t version)
 {
     RefPtr<IDBOpenDBRequest> request;
     {
-        Locker<Lock> locker(m_openDBRequestMapLock);
+        Locker locker { m_openDBRequestMapLock };
 
         request = IDBOpenDBRequest::createOpenRequest(context, *this, databaseIdentifier, version);
         ASSERT(!m_openDBRequestMap.contains(request->resourceIdentifier()));
         m_openDBRequestMap.set(request->resourceIdentifier(), request.get());
     }
 
-    callConnectionOnMainThread(&IDBConnectionToServer::openDatabase, IDBRequestData(*this, *request));
+    callConnectionOnMainThread(&IDBConnectionToServer::openDatabase, IDBOpenRequestData(*this, *request));
 
     return request.releaseNonNull();
 }
@@ -80,14 +90,14 @@ Ref<IDBOpenDBRequest> IDBConnectionProxy::deleteDatabase(ScriptExecutionContext&
 {
     RefPtr<IDBOpenDBRequest> request;
     {
-        Locker<Lock> locker(m_openDBRequestMapLock);
+        Locker locker { m_openDBRequestMapLock };
 
         request = IDBOpenDBRequest::createDeleteRequest(context, *this, databaseIdentifier);
         ASSERT(!m_openDBRequestMap.contains(request->resourceIdentifier()));
         m_openDBRequestMap.set(request->resourceIdentifier(), request.get());
     }
 
-    callConnectionOnMainThread(&IDBConnectionToServer::deleteDatabase, IDBRequestData(*this, *request));
+    callConnectionOnMainThread(&IDBConnectionToServer::deleteDatabase, IDBOpenRequestData(*this, *request));
 
     return request.releaseNonNull();
 }
@@ -108,7 +118,7 @@ void IDBConnectionProxy::completeOpenDBRequest(const IDBResultData& resultData)
 
     RefPtr<IDBOpenDBRequest> request;
     {
-        Locker<Lock> locker(m_openDBRequestMapLock);
+        Locker locker { m_openDBRequestMapLock };
         request = m_openDBRequestMap.take(resultData.requestIdentifier());
     }
 
@@ -119,7 +129,7 @@ void IDBConnectionProxy::completeOpenDBRequest(const IDBResultData& resultData)
         switch (resultData.type()) {
         case IDBResultType::OpenDatabaseUpgradeNeeded: {
             abortOpenAndUpgradeNeeded(resultData.databaseConnectionIdentifier(), resultData.transactionInfo().identifier());
-            auto result = IDBResultData::error(resultData.requestIdentifier(), IDBError { UnknownError, "Version change transaction on cached page is aborted to unblock other connections"_s });
+            auto result = IDBResultData::error(resultData.requestIdentifier(), IDBError { ExceptionCode::UnknownError, "Version change transaction on cached page is aborted to unblock other connections"_s });
             request->performCallbackOnOriginThread(*request, &IDBOpenDBRequest::requestCompleted, result);
             return;
         }
@@ -139,7 +149,7 @@ void IDBConnectionProxy::createObjectStore(TransactionOperation& operation, cons
     callConnectionOnMainThread(&IDBConnectionToServer::createObjectStore, requestData, info);
 }
 
-void IDBConnectionProxy::renameObjectStore(TransactionOperation& operation, uint64_t objectStoreIdentifier, const String& newName)
+void IDBConnectionProxy::renameObjectStore(TransactionOperation& operation, IDBObjectStoreIdentifier objectStoreIdentifier, const String& newName)
 {
     const IDBRequestData requestData { operation };
     saveOperation(operation);
@@ -147,7 +157,7 @@ void IDBConnectionProxy::renameObjectStore(TransactionOperation& operation, uint
     callConnectionOnMainThread(&IDBConnectionToServer::renameObjectStore, requestData, objectStoreIdentifier, newName);
 }
 
-void IDBConnectionProxy::renameIndex(TransactionOperation& operation, uint64_t objectStoreIdentifier, uint64_t indexIdentifier, const String& newName)
+void IDBConnectionProxy::renameIndex(TransactionOperation& operation, IDBObjectStoreIdentifier objectStoreIdentifier, IDBIndexIdentifier indexIdentifier, const String& newName)
 {
     const IDBRequestData requestData { operation };
     saveOperation(operation);
@@ -163,7 +173,7 @@ void IDBConnectionProxy::deleteObjectStore(TransactionOperation& operation, cons
     callConnectionOnMainThread(&IDBConnectionToServer::deleteObjectStore, requestData, objectStoreName);
 }
 
-void IDBConnectionProxy::clearObjectStore(TransactionOperation& operation, uint64_t objectStoreIdentifier)
+void IDBConnectionProxy::clearObjectStore(TransactionOperation& operation, IDBObjectStoreIdentifier objectStoreIdentifier)
 {
     const IDBRequestData requestData { operation };
     saveOperation(operation);
@@ -179,7 +189,7 @@ void IDBConnectionProxy::createIndex(TransactionOperation& operation, const IDBI
     callConnectionOnMainThread(&IDBConnectionToServer::createIndex, requestData, info);
 }
 
-void IDBConnectionProxy::deleteIndex(TransactionOperation& operation, uint64_t objectStoreIdentifier, const String& indexName)
+void IDBConnectionProxy::deleteIndex(TransactionOperation& operation, IDBObjectStoreIdentifier objectStoreIdentifier, const String& indexName)
 {
     const IDBRequestData requestData { operation };
     saveOperation(operation);
@@ -187,12 +197,12 @@ void IDBConnectionProxy::deleteIndex(TransactionOperation& operation, uint64_t o
     callConnectionOnMainThread(&IDBConnectionToServer::deleteIndex, requestData, WTFMove(objectStoreIdentifier), indexName);
 }
 
-void IDBConnectionProxy::putOrAdd(TransactionOperation& operation, IDBKeyData&& keyData, const IDBValue& value, const IndexedDB::ObjectStoreOverwriteMode mode)
+void IDBConnectionProxy::putOrAdd(TransactionOperation& operation, IDBKeyData&& keyData, const IDBValue& value, const IndexIDToIndexKeyMap& indexKeys, const IndexedDB::ObjectStoreOverwriteMode mode)
 {
     const IDBRequestData requestData { operation };
     saveOperation(operation);
 
-    callConnectionOnMainThread(&IDBConnectionToServer::putOrAdd, requestData, keyData, value, mode);
+    callConnectionOnMainThread(&IDBConnectionToServer::putOrAdd, requestData, keyData, value, indexKeys, mode);
 }
 
 void IDBConnectionProxy::getRecord(TransactionOperation& operation, const IDBGetRecordData& getRecordData)
@@ -246,7 +256,7 @@ void IDBConnectionProxy::iterateCursor(TransactionOperation& operation, const ID
 
 void IDBConnectionProxy::saveOperation(TransactionOperation& operation)
 {
-    Locker<Lock> locker(m_transactionOperationLock);
+    Locker locker { m_transactionOperationLock };
 
     ASSERT(!m_activeOperations.contains(operation.identifier()));
     m_activeOperations.set(operation.identifier(), &operation);
@@ -256,7 +266,7 @@ void IDBConnectionProxy::completeOperation(const IDBResultData& resultData)
 {
     RefPtr<TransactionOperation> operation;
     {
-        Locker<Lock> locker(m_transactionOperationLock);
+        Locker locker { m_transactionOperationLock };
         operation = m_activeOperations.take(resultData.requestIdentifier());
     }
 
@@ -266,43 +276,74 @@ void IDBConnectionProxy::completeOperation(const IDBResultData& resultData)
     operation->transitionToComplete(resultData, WTFMove(operation));
 }
 
-void IDBConnectionProxy::abortOpenAndUpgradeNeeded(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& transactionIdentifier)
+void IDBConnectionProxy::abortOpenAndUpgradeNeeded(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const std::optional<IDBResourceIdentifier>& transactionIdentifier)
 {
     callConnectionOnMainThread(&IDBConnectionToServer::abortOpenAndUpgradeNeeded, databaseConnectionIdentifier, transactionIdentifier);
 }
 
-void IDBConnectionProxy::fireVersionChangeEvent(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, uint64_t requestedVersion)
+std::pair<RefPtr<IDBDatabase>, std::optional<ScriptExecutionContextIdentifier>> IDBConnectionProxy::databaseFromConnectionIdentifier(IDBDatabaseConnectionIdentifier connectionIdentifier)
 {
-    RefPtr<IDBDatabase> database;
-    {
-        Locker<Lock> locker(m_databaseConnectionMapLock);
-        database = m_databaseConnectionMap.get(databaseConnectionIdentifier);
+    Locker locker { m_databaseConnectionMapLock };
+    auto it = m_databaseConnectionMap.find(connectionIdentifier);
+    if (it == m_databaseConnectionMap.end())
+        return { };
+
+    RefPtr database = it->value.database.get();
+    if (!database) {
+        m_databaseConnectionMap.remove(it);
+        return { };
     }
 
+    return { database, it->value.contextIdentifier };
+}
+
+void IDBConnectionProxy::fireVersionChangeEvent(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, uint64_t requestedVersion)
+{
+    auto [database, contextIdentifier] = databaseFromConnectionIdentifier(databaseConnectionIdentifier);
     if (!database)
         return;
 
+    ASSERT(contextIdentifier);
     if (database->isContextSuspended()) {
         didFireVersionChangeEvent(databaseConnectionIdentifier, requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer::Yes);
-        database->performCallbackOnOriginThread(*database, &IDBDatabase::connectionToServerLost, IDBError { UnknownError, "Connection on cached page closed to unblock other connections"_s});
+        ScriptExecutionContext::ensureOnContextThreadForCrossThreadTask(*contextIdentifier, createCrossThreadTask(*database,  &IDBDatabase::connectionToServerLost, IDBError { ExceptionCode::UnknownError, "Connection on cached page closed to unblock other connections"_s }));
         return;
     }
 
-    database->performCallbackOnOriginThread(*database, &IDBDatabase::fireVersionChangeEvent, requestIdentifier, requestedVersion);
+    ScriptExecutionContext::ensureOnContextThreadForCrossThreadTask(*contextIdentifier, createCrossThreadTask(*database,  &IDBDatabase::fireVersionChangeEvent, requestIdentifier, requestedVersion));
 }
 
-void IDBConnectionProxy::didFireVersionChangeEvent(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer connectionClosed)
+void IDBConnectionProxy::didFireVersionChangeEvent(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const IDBResourceIdentifier& requestIdentifier, IndexedDB::ConnectionClosedOnBehalfOfServer connectionClosed)
 {
     callConnectionOnMainThread(&IDBConnectionToServer::didFireVersionChangeEvent, databaseConnectionIdentifier, requestIdentifier, connectionClosed);
 }
 
+void IDBConnectionProxy::generateIndexKeyForRecord(const IDBResourceIdentifier& requestIdentifier, const IDBIndexInfo& indexInfo, const std::optional<IDBKeyPath>& keyPath, const IDBKeyData& key, const IDBValue& value, std::optional<int64_t> recordID)
+{
+    Locker locker { m_transactionOperationLock };
+    RefPtr operation = m_activeOperations.get(requestIdentifier);
+    if (!operation)
+        return;
+
+    auto contextIdentifier = operation->scriptExecutionContextIdentifier();
+    if (!contextIdentifier)
+        return;
+
+    ScriptExecutionContext::ensureOnContextThreadForCrossThreadTask(*contextIdentifier, createCrossThreadTask(operation->transaction(),  &IDBTransaction::generateIndexKeyForRecord, requestIdentifier, indexInfo, keyPath, key, value, recordID));
+}
+
+void IDBConnectionProxy::didGenerateIndexKeyForRecord(const IDBResourceIdentifier& transactionIdentifier, const IDBResourceIdentifier& requestIdentifier, const IDBIndexInfo& indexInfo, const IDBKeyData& key, const IndexKey& indexKey, std::optional<int64_t> recordID)
+{
+    callConnectionOnMainThread(&IDBConnectionToServer::didGenerateIndexKeyForRecord, transactionIdentifier, requestIdentifier, indexInfo, key, indexKey, recordID);
+}
+
 void IDBConnectionProxy::notifyOpenDBRequestBlocked(const IDBResourceIdentifier& requestIdentifier, uint64_t oldVersion, uint64_t newVersion)
 {
     ASSERT(isMainThread());
 
     RefPtr<IDBOpenDBRequest> request;
     {
-        Locker<Lock> locker(m_openDBRequestMapLock);
+        Locker locker { m_openDBRequestMapLock };
         request = m_openDBRequestMap.get(requestIdentifier);
     }
 
@@ -312,7 +353,7 @@ void IDBConnectionProxy::notifyOpenDBRequestBlocked(const IDBResourceIdentifier&
     request->performCallbackOnOriginThread(*request, &IDBOpenDBRequest::requestBlocked, oldVersion, newVersion);
 }
 
-void IDBConnectionProxy::openDBRequestCancelled(const IDBRequestData& requestData)
+void IDBConnectionProxy::openDBRequestCancelled(const IDBOpenRequestData& requestData)
 {
     callConnectionOnMainThread(&IDBConnectionToServer::openDBRequestCancelled, requestData);
 }
@@ -320,7 +361,7 @@ void IDBConnectionProxy::openDBRequestCancelled(const IDBRequestData& requestDat
 void IDBConnectionProxy::establishTransaction(IDBTransaction& transaction)
 {
     {
-        Locker<Lock> locker(m_transactionMapLock);
+        Locker locker { m_transactionMapLock };
         ASSERT(!hasRecordOfTransaction(transaction));
         m_pendingTransactions.set(transaction.info().identifier(), &transaction);
     }
@@ -332,7 +373,7 @@ void IDBConnectionProxy::didStartTransaction(const IDBResourceIdentifier& transa
 {
     RefPtr<IDBTransaction> transaction;
     {
-        Locker<Lock> locker(m_transactionMapLock);
+        Locker locker { m_transactionMapLock };
         transaction = m_pendingTransactions.take(transactionIdentifier);
     }
 
@@ -342,22 +383,22 @@ void IDBConnectionProxy::didStartTransaction(const IDBResourceIdentifier& transa
     transaction->performCallbackOnOriginThread(*transaction, &IDBTransaction::didStart, error);
 }
 
-void IDBConnectionProxy::commitTransaction(IDBTransaction& transaction)
+void IDBConnectionProxy::commitTransaction(IDBTransaction& transaction, uint64_t handledRequestResultsCount)
 {
     {
-        Locker<Lock> locker(m_transactionMapLock);
+        Locker locker { m_transactionMapLock };
         ASSERT(!m_committingTransactions.contains(transaction.info().identifier()));
         m_committingTransactions.set(transaction.info().identifier(), &transaction);
     }
 
-    callConnectionOnMainThread(&IDBConnectionToServer::commitTransaction, transaction.info().identifier());
+    callConnectionOnMainThread(&IDBConnectionToServer::commitTransaction, transaction.info().identifier(), handledRequestResultsCount);
 }
 
 void IDBConnectionProxy::didCommitTransaction(const IDBResourceIdentifier& transactionIdentifier, const IDBError& error)
 {
     RefPtr<IDBTransaction> transaction;
     {
-        Locker<Lock> locker(m_transactionMapLock);
+        Locker locker { m_transactionMapLock };
         transaction = m_committingTransactions.take(transactionIdentifier);
     }
 
@@ -370,7 +411,7 @@ void IDBConnectionProxy::didCommitTransaction(const IDBResourceIdentifier& trans
 void IDBConnectionProxy::abortTransaction(IDBTransaction& transaction)
 {
     {
-        Locker<Lock> locker(m_transactionMapLock);
+        Locker locker { m_transactionMapLock };
         ASSERT(!m_abortingTransactions.contains(transaction.info().identifier()));
         m_abortingTransactions.set(transaction.info().identifier(), &transaction);
     }
@@ -382,7 +423,7 @@ void IDBConnectionProxy::didAbortTransaction(const IDBResourceIdentifier& transa
 {
     RefPtr<IDBTransaction> transaction;
     {
-        Locker<Lock> locker(m_transactionMapLock);
+        Locker locker { m_transactionMapLock };
         transaction = m_abortingTransactions.take(transactionIdentifier);
     }
 
@@ -400,7 +441,7 @@ bool IDBConnectionProxy::hasRecordOfTransaction(const IDBTransaction& transactio
     return m_pendingTransactions.contains(identifier) || m_committingTransactions.contains(identifier) || m_abortingTransactions.contains(identifier);
 }
 
-void IDBConnectionProxy::didFinishHandlingVersionChangeTransaction(uint64_t databaseConnectionIdentifier, IDBTransaction& transaction)
+void IDBConnectionProxy::didFinishHandlingVersionChangeTransaction(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, IDBTransaction& transaction)
 {
     callConnectionOnMainThread(&IDBConnectionToServer::didFinishHandlingVersionChangeTransaction, databaseConnectionIdentifier, transaction.info().identifier());
 }
@@ -415,51 +456,41 @@ void IDBConnectionProxy::databaseConnectionClosed(IDBDatabase& database)
     callConnectionOnMainThread(&IDBConnectionToServer::databaseConnectionClosed, database.databaseConnectionIdentifier());
 }
 
-void IDBConnectionProxy::didCloseFromServer(uint64_t databaseConnectionIdentifier, const IDBError& error)
+void IDBConnectionProxy::didCloseFromServer(IDBDatabaseConnectionIdentifier databaseConnectionIdentifier, const IDBError& error)
 {
-    RefPtr<IDBDatabase> database;
-    {
-        Locker<Lock> locker(m_databaseConnectionMapLock);
-        database = m_databaseConnectionMap.get(databaseConnectionIdentifier);
-    }
-
+    auto [database, contextIdentifier] = databaseFromConnectionIdentifier(databaseConnectionIdentifier);
     if (!database)
         return;
 
-    database->performCallbackOnOriginThread(*database, &IDBDatabase::didCloseFromServer, error);
+    ScriptExecutionContext::ensureOnContextThreadForCrossThreadTask(*contextIdentifier, createCrossThreadTask(*database,  &IDBDatabase::didCloseFromServer, error));
 }
 
 void IDBConnectionProxy::connectionToServerLost(const IDBError& error)
 {
-    Vector<uint64_t> databaseConnectionIdentifiers;
+    Vector<IDBDatabaseConnectionIdentifier> databaseConnectionIdentifiers;
     {
-        Locker<Lock> locker(m_databaseConnectionMapLock);
+        Locker locker { m_databaseConnectionMapLock };
         databaseConnectionIdentifiers = copyToVector(m_databaseConnectionMap.keys());
     }
 
     for (auto connectionIdentifier : databaseConnectionIdentifiers) {
-        RefPtr<IDBDatabase> database;
-        {
-            Locker<Lock> locker(m_databaseConnectionMapLock);
-            database = m_databaseConnectionMap.get(connectionIdentifier);
-        }
-
+        auto [database, contextIdentifier] = databaseFromConnectionIdentifier(connectionIdentifier);
         if (!database)
             continue;
 
-        database->performCallbackOnOriginThread(*database, &IDBDatabase::connectionToServerLost, error);
+        ScriptExecutionContext::ensureOnContextThreadForCrossThreadTask(*contextIdentifier, createCrossThreadTask(*database,  &IDBDatabase::connectionToServerLost, error));
     }
 
     Vector<IDBResourceIdentifier> openDBRequestIdentifiers;
     {
-        Locker<Lock> locker(m_openDBRequestMapLock);
+        Locker locker { m_openDBRequestMapLock };
         openDBRequestIdentifiers = copyToVector(m_openDBRequestMap.keys());
     }
 
     for (auto& requestIdentifier : openDBRequestIdentifiers) {
         RefPtr<IDBOpenDBRequest> request;
         {
-            Locker<Lock> locker(m_openDBRequestMapLock);
+            Locker locker { m_openDBRequestMapLock };
             request = m_openDBRequestMap.get(requestIdentifier);
         }
 
@@ -469,15 +500,24 @@ void IDBConnectionProxy::connectionToServerLost(const IDBError& error)
         auto result = IDBResultData::error(requestIdentifier, error);
         request->performCallbackOnOriginThread(*request, &IDBOpenDBRequest::requestCompleted, result);
     }
+
+    Vector<IDBResourceIdentifier> infoCallbackIdentifiers;
+    {
+        Locker locker { m_databaseInfoMapLock };
+        infoCallbackIdentifiers = copyToVector(m_databaseInfoCallbacks.keys());
+    }
+
+    for (auto& requestIdentifier : infoCallbackIdentifiers)
+        didGetAllDatabaseNamesAndVersions(requestIdentifier, { });
 }
 
 void IDBConnectionProxy::scheduleMainThreadTasks()
 {
-    Locker<Lock> locker(m_mainThreadTaskLock);
+    Locker locker { m_mainThreadTaskLock };
     if (m_mainThreadProtector)
         return;
 
-    m_mainThreadProtector = &m_connectionToServer;
+    m_mainThreadProtector = m_connectionToServer.ptr();
     callOnMainThread([this] {
         handleMainThreadTasks();
     });
@@ -487,7 +527,7 @@ void IDBConnectionProxy::handleMainThreadTasks()
 {
     RefPtr<IDBConnectionToServer> protector;
     {
-        Locker<Lock> locker(m_mainThreadTaskLock);
+        Locker locker { m_mainThreadTaskLock };
         ASSERT(m_mainThreadProtector);
         protector = WTFMove(m_mainThreadProtector);
     }
@@ -496,33 +536,53 @@ void IDBConnectionProxy::handleMainThreadTasks()
         task->performTask();
 }
 
-void IDBConnectionProxy::getAllDatabaseNames(const SecurityOrigin& mainFrameOrigin, const SecurityOrigin& openingOrigin, Function<void (const Vector<String>&)>&& callback)
+void IDBConnectionProxy::getAllDatabaseNamesAndVersions(ScriptExecutionContext& context, Function<void(std::optional<Vector<IDBDatabaseNameAndVersion>>&&)>&& callback)
 {
-    // This method is only meant to be called by the web inspector on the main thread.
-    RELEASE_ASSERT(isMainThread());
+    ClientOrigin origin { context.securityOrigin()->data(), context.topOrigin().data() };
 
-    m_connectionToServer.getAllDatabaseNames(mainFrameOrigin, openingOrigin, WTFMove(callback));
+    IDBDatabaseNameAndVersionRequest* request;
+    {
+        Locker locker { m_databaseInfoMapLock };
+        auto newRequest = IDBDatabaseNameAndVersionRequest::create(context, *this, WTFMove(callback));
+        ASSERT(!m_databaseInfoCallbacks.contains(newRequest->resourceIdentifier()));
+        request = newRequest.ptr();
+        m_databaseInfoCallbacks.add(newRequest->resourceIdentifier(), WTFMove(newRequest));
+    }
+
+    callConnectionOnMainThread(&IDBConnectionToServer::getAllDatabaseNamesAndVersions, request->resourceIdentifier(), origin);
 }
 
-void IDBConnectionProxy::registerDatabaseConnection(IDBDatabase& database)
+void IDBConnectionProxy::didGetAllDatabaseNamesAndVersions(const IDBResourceIdentifier& requestIdentifier, std::optional<Vector<IDBDatabaseNameAndVersion>>&& databases)
 {
-    Locker<Lock> locker(m_databaseConnectionMapLock);
+    RefPtr<IDBDatabaseNameAndVersionRequest> request;
+    {
+        Locker locker { m_databaseInfoMapLock };
+        request = m_databaseInfoCallbacks.take(requestIdentifier);
+        if (!request)
+            return;
+    }
+
+    request->performCallbackOnOriginThread(*request, &IDBDatabaseNameAndVersionRequest::complete, WTFMove(databases));
+}
+
+void IDBConnectionProxy::registerDatabaseConnection(IDBDatabase& database, ScriptExecutionContextIdentifier identifier)
+{
+    Locker locker { m_databaseConnectionMapLock };
 
     ASSERT(!m_databaseConnectionMap.contains(database.databaseConnectionIdentifier()));
-    m_databaseConnectionMap.set(database.databaseConnectionIdentifier(), &database);
+    m_databaseConnectionMap.set(database.databaseConnectionIdentifier(), WeakIDBDatabase { &database, identifier });
 }
 
 void IDBConnectionProxy::unregisterDatabaseConnection(IDBDatabase& database)
 {
-    Locker<Lock> locker(m_databaseConnectionMapLock);
+    Locker locker { m_databaseConnectionMapLock };
 
-    ASSERT(!m_databaseConnectionMap.contains(database.databaseConnectionIdentifier()) || m_databaseConnectionMap.get(database.databaseConnectionIdentifier()) == &database);
     m_databaseConnectionMap.remove(database.databaseConnectionIdentifier());
 }
 
 void IDBConnectionProxy::forgetActiveOperations(const Vector<RefPtr<TransactionOperation>>& operations)
 {
-    Locker<Lock> locker(m_transactionOperationLock);
+    Locker locker { m_transactionOperationLock };
 
     for (auto& operation : operations)
         m_activeOperations.remove(operation->identifier());
@@ -530,29 +590,25 @@ void IDBConnectionProxy::forgetActiveOperations(const Vector<RefPtr<TransactionO
 
 void IDBConnectionProxy::forgetTransaction(IDBTransaction& transaction)
 {
-    Locker<Lock> locker(m_transactionMapLock);
+    Locker locker { m_transactionMapLock };
 
     m_pendingTransactions.remove(transaction.info().identifier());
     m_committingTransactions.remove(transaction.info().identifier());
     m_abortingTransactions.remove(transaction.info().identifier());
 }
 
-template<typename KeyType, typename ValueType>
-void removeItemsMatchingCurrentThread(HashMap<KeyType, ValueType>& map)
+template<typename KeyType, typename ValueType, typename FunctionType>
+void removeItemsMatchingCurrentThread(HashMap<KeyType, ValueType>& map, FunctionType&& cleanupFunction)
 {
     // FIXME: Revisit when introducing WebThread aware thread comparison.
     // https://bugs.webkit.org/show_bug.cgi?id=204345
-    auto& currentThread = Thread::current();
-
-    Vector<KeyType> keys;
-    keys.reserveInitialCapacity(map.size());
-    for (auto& iterator : map) {
-        if (&iterator.value->originThread() == &currentThread)
-            keys.uncheckedAppend(iterator.key);
-    }
-
-    for (auto& key : keys)
-        map.remove(key);
+    map.removeIf([currentThread = &Thread::currentSingleton(), cleanupFunction = WTFMove(cleanupFunction)](auto& entry) {
+        if (&entry.value->originThread() == currentThread) {
+            cleanupFunction(entry.value);
+            return true;
+        }
+        return false;
+    });
 }
 
 template<typename KeyType, typename ValueType>
@@ -560,12 +616,12 @@ void setMatchingItemsContextSuspended(ScriptExecutionContext& currentContext, Ha
 {
     // FIXME: Revisit when introducing WebThread aware thread comparison.
     // https://bugs.webkit.org/show_bug.cgi?id=204345
-    auto& currentThread = Thread::current();
+    auto& currentThread = Thread::currentSingleton();
     for (auto& iterator : map) {
         if (&iterator.value->originThread() != &currentThread)
             continue;
 
-        auto* context = iterator.value->scriptExecutionContext();
+        RefPtr context = iterator.value->scriptExecutionContext();
         if (!context)
             continue;
 
@@ -574,43 +630,74 @@ void setMatchingItemsContextSuspended(ScriptExecutionContext& currentContext, Ha
     }
 }
 
-void IDBConnectionProxy::forgetActivityForCurrentThread()
+void IDBConnectionProxy::abortActivitiesForCurrentThread()
 {
-    ASSERT(!isMainThread());
+    {
+        Locker locker { m_openDBRequestMapLock };
+        removeItemsMatchingCurrentThread(m_openDBRequestMap, [](RefPtr<IDBOpenDBRequest>& request) {
+            if (request)
+                request->requestCompleted(IDBResultData::error(request->resourceIdentifier(), IDBError { ExceptionCode::InvalidStateError, "Request is removed"_s }));
+        });
+    }
+    {
+        Locker locker { m_transactionMapLock };
+        removeItemsMatchingCurrentThread(m_pendingTransactions, [](auto& transaction) {
+            if (transaction)
+                transaction->didStart(IDBError { ExceptionCode::InvalidStateError, "Transaction is removed"_s });
+        });
+        removeItemsMatchingCurrentThread(m_committingTransactions, [](auto& transaction) {
+            if (transaction)
+                transaction->didCommit(IDBError { ExceptionCode::InvalidStateError, "Transaction is removed"_s });
+        });
+        removeItemsMatchingCurrentThread(m_abortingTransactions, [](auto& transaction) {
+            if (transaction)
+                transaction->didAbort(IDBError { ExceptionCode::InvalidStateError, "Transaction is removed"_s });
+        });
+    }
+
+    HashSet<RefPtr<TransactionOperation>> activeOperationsForThread;
+    {
+        Locker locker { m_transactionOperationLock };
+        for (auto& operation : m_activeOperations.values()) {
+            if (&operation->originThread() == &Thread::currentSingleton())
+                activeOperationsForThread.add(operation);
+        }
+        for (auto& operation : activeOperationsForThread)
+            m_activeOperations.remove(operation->identifier());
+    }
+
+    for (auto& operation : activeOperationsForThread) {
+        auto result = IDBResultData::error(operation->identifier(), IDBError { ExceptionCode::InvalidStateError, "Operation is removed"_s });
+        operation->transitionToCompleteOnThisThread(result);
+    }
 
     {
-        Locker<Lock> lock(m_databaseConnectionMapLock);
-        removeItemsMatchingCurrentThread(m_databaseConnectionMap);
-    }
-    {
-        Locker<Lock> lock(m_openDBRequestMapLock);
-        removeItemsMatchingCurrentThread(m_openDBRequestMap);
-    }
-    {
-        Locker<Lock> lock(m_transactionMapLock);
-        removeItemsMatchingCurrentThread(m_pendingTransactions);
-        removeItemsMatchingCurrentThread(m_committingTransactions);
-        removeItemsMatchingCurrentThread(m_abortingTransactions);
-    }
-    {
-        Locker<Lock> lock(m_transactionOperationLock);
-        removeItemsMatchingCurrentThread(m_activeOperations);
+        Locker locker { m_databaseInfoMapLock };
+        removeItemsMatchingCurrentThread(m_databaseInfoCallbacks, [](auto& request) {
+            request->complete(std::nullopt);
+        });
     }
 }
 
 void IDBConnectionProxy::setContextSuspended(ScriptExecutionContext& currentContext, bool isContextSuspended)
 {
     {
-        Locker<Lock> lock(m_databaseConnectionMapLock);
-        setMatchingItemsContextSuspended(currentContext, m_databaseConnectionMap, isContextSuspended);
+        auto currentContextIdentifier = currentContext.identifier();
+        Locker locker { m_databaseConnectionMapLock };
+        for (auto weakDatabase : m_databaseConnectionMap.values()) {
+            if (RefPtr database = weakDatabase.database.get()) {
+                auto contextIdentifier = weakDatabase.contextIdentifier;
+                if (contextIdentifier && contextIdentifier == currentContextIdentifier)
+                    database->setIsContextSuspended(isContextSuspended);
+            }
+        }
     }
+
     {
-        Locker<Lock> lock(m_openDBRequestMapLock);
+        Locker locker { m_openDBRequestMapLock };
         setMatchingItemsContextSuspended(currentContext, m_openDBRequestMap, isContextSuspended);
     }
 }
 
 } // namesapce IDBClient
 } // namespace WebCore
-
-#endif // ENABLE(INDEXED_DATABASE)
