@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010, Google Inc. All rights reserved.
+ * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,7 +32,7 @@ namespace WebCore {
 class AudioContext;
     
 class ChannelSplitterNode final : public AudioNode {
-    WTF_MAKE_ISO_ALLOCATED(ChannelSplitterNode);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(ChannelSplitterNode);
 public:
     static ExceptionOr<Ref<ChannelSplitterNode>> create(BaseAudioContext&, const ChannelSplitterOptions& = { });
 
