@@ -109,7 +109,7 @@ uint32_t DFANode::bestFallbackTarget(const DFA& dfa) const
 {
     ASSERT(canUseFallbackTransition(dfa));
 
-    UncheckedKeyHashMap<uint32_t, unsigned, DefaultHash<uint32_t>, WTF::UnsignedWithZeroKeyHashTraits<uint32_t>> histogram;
+    HashMap<uint32_t, unsigned, DefaultHash<uint32_t>, WTF::UnsignedWithZeroKeyHashTraits<uint32_t>> histogram;
 
     IterableConstRange iterableTransitions = transitions(dfa);
     auto iterator = iterableTransitions.begin();
