@@ -30,18 +30,23 @@
 
 #pragma once
 
-#if ENABLE(INPUT_TYPE_DATE)
-
 #include "BaseDateAndTimeInputType.h"
+#include <wtf/TZoneMalloc.h>
 
 namespace WebCore {
 
 class DateInputType final : public BaseDateAndTimeInputType {
-    template<typename DowncastedType> friend bool isInvalidInputType(const InputType&, const String&);
+    WTF_MAKE_TZONE_ALLOCATED(DateInputType);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(DateInputType);
 public:
-    explicit DateInputType(HTMLInputElement&);
+    static Ref<DateInputType> create(HTMLInputElement& element)
+    {
+        return adoptRef(*new DateInputType(element));
+    }
 
 private:
+    explicit DateInputType(HTMLInputElement&);
+
     const AtomString& formControlType() const final;
     DateComponentsType dateType() const final;
     StepRange createStepRange(AnyStepHandling) const final;
@@ -56,5 +61,3 @@ private:
 } // namespace WebCore
 
 SPECIALIZE_TYPE_TRAITS_INPUT_TYPE(DateInputType, Type::Date)
-
-#endif
