@@ -30,8 +30,8 @@
 
 #include "ContextMenu.h"
 #include "ContextMenuProvider.h"
-#include "ExceptionOr.h"
 #include "InspectorFrontendClient.h"
+#include <JavaScriptCore/JSCJSValue.h>
 #include <wtf/RefCounted.h>
 #include <wtf/Vector.h>
 #include <wtf/text/WTFString.h>
@@ -46,8 +46,10 @@ class Event;
 class File;
 class FrontendMenuProvider;
 class HTMLIFrameElement;
+class OffscreenCanvasRenderingContext2D;
 class Page;
 class Path2D;
+template<typename> class ExceptionOr;
 
 class InspectorFrontendHost : public RefCounted<InspectorFrontendHost> {
 public:
@@ -145,6 +147,8 @@ public:
     bool isBeingInspected();
     void setAllowsInspectingInspector(bool);
 
+    bool engineeringSettingsAllowed();
+
     bool supportsDiagnosticLogging();
 #if ENABLE(INSPECTOR_TELEMETRY)
     bool diagnosticLoggingAvailable();
@@ -169,6 +173,13 @@ public:
     Ref<Path2D> getPath(const CanvasRenderingContext2D&) const;
     void setPath(CanvasRenderingContext2D&, Path2D&) const;
 
+#if ENABLE(OFFSCREEN_CANVAS)
+    float getCurrentX(const OffscreenCanvasRenderingContext2D&) const;
+    float getCurrentY(const OffscreenCanvasRenderingContext2D&) const;
+    Ref<Path2D> getPath(const OffscreenCanvasRenderingContext2D&) const;
+    void setPath(OffscreenCanvasRenderingContext2D&, Path2D&) const;
+#endif
+
 private:
 #if ENABLE(CONTEXT_MENUS)
     friend class FrontendMenuProvider;
@@ -176,7 +187,7 @@ private:
     WEBCORE_EXPORT InspectorFrontendHost(InspectorFrontendClient*, Page* frontendPage);
 
     InspectorFrontendClient* m_client;
-    Page* m_frontendPage;
+    WeakPtr<Page> m_frontendPage;
 #if ENABLE(CONTEXT_MENUS)
     FrontendMenuProvider* m_menuProvider;
 #endif
