@@ -29,6 +29,7 @@
 
 #include "ImageOrientation.h"
 #include "VideoFrame.h"
+#include <wtf/ArgumentCoder.h>
 #include <wtf/RetainPtr.h>
 
 using CMSampleBufferRef = struct opaqueCMSampleBuffer*;
@@ -39,45 +40,29 @@ class PixelBuffer;
 
 class VideoFrameCV : public VideoFrame {
 public:
-    WEBCORE_EXPORT static Ref<VideoFrameCV> create(MediaTime presentationTime, bool isMirrored, Rotation, RetainPtr<CVPixelBufferRef>&&);
+    WEBCORE_EXPORT static Ref<VideoFrameCV> create(MediaTime presentationTime, bool isMirrored, Rotation, RetainPtr<CVPixelBufferRef>&&, std::optional<PlatformVideoColorSpace>&& = { });
     WEBCORE_EXPORT static Ref<VideoFrameCV> create(CMSampleBufferRef, bool isMirrored, Rotation);
-    static RefPtr<VideoFrameCV> createFromPixelBuffer(Ref<PixelBuffer>&&);
     WEBCORE_EXPORT ~VideoFrameCV();
 
     CVPixelBufferRef pixelBuffer() const final { return m_pixelBuffer.get(); }
     ImageOrientation orientation() const;
 
-    template<typename Encoder> void encode(Encoder&) const;
-    template<typename Decoder> static std::optional<RefPtr<VideoFrameCV>> decode(Decoder&);
-
     // VideoFrame overrides.
-    WEBCORE_EXPORT WebCore::FloatSize presentationSize() const final;
+    WEBCORE_EXPORT WebCore::IntSize presentationSize() const final;
     WEBCORE_EXPORT uint32_t pixelFormat() const final;
     WEBCORE_EXPORT void setOwnershipIdentity(const ProcessIdentity&) final;
     bool isCV() const final { return true; }
 
 private:
-    WEBCORE_EXPORT VideoFrameCV(MediaTime presentationTime, bool isMirrored, Rotation, RetainPtr<CVPixelBufferRef>&&);
+    friend struct IPC::ArgumentCoder<VideoFrameCV, void>;
+    WEBCORE_EXPORT VideoFrameCV(MediaTime presentationTime, bool isMirrored, Rotation, RetainPtr<CVPixelBufferRef>&&, std::optional<PlatformVideoColorSpace>&&);
+    VideoFrameCV(MediaTime presentationTime, bool isMirrored, Rotation, RetainPtr<CVPixelBufferRef>&&, PlatformVideoColorSpace&&);
+
+    Ref<VideoFrame> clone() final;
 
     const RetainPtr<CVPixelBufferRef> m_pixelBuffer;
 };
 
-template<typename Encoder> void VideoFrameCV::encode(Encoder& encoder) const
-{
-    encoder << presentationTime() << isMirrored() << rotation() << m_pixelBuffer;
-}
-
-template<typename Decoder> std::optional<RefPtr<VideoFrameCV>> VideoFrameCV::decode(Decoder& decoder)
-{
-    auto presentationTime = decoder.template decode<MediaTime>();
-    auto isMirrored = decoder.template decode<bool>();
-    auto rotation = decoder.template decode<Rotation>();
-    auto pixelBuffer = decoder.template decode<RetainPtr<CVPixelBufferRef>>();
-    if (!decoder.isValid() || !*pixelBuffer)
-        return std::nullopt;
-    return VideoFrameCV::create(*presentationTime, *isMirrored, *rotation, pixelBuffer.releaseNonNull());
-}
-
 }
 
 SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::VideoFrameCV)
