@@ -51,10 +51,21 @@ public:
         StartingStyle = 1 << 5,
         NonCacheable = 1 << 6,
     };
-    static constexpr OptionSet<PropertyType> normalProperties() { return { PropertyType::NonInherited,  PropertyType::Inherited }; }
-    static constexpr OptionSet<PropertyType> startingStyleProperties() { return normalProperties() | PropertyType::StartingStyle; }
 
-    PropertyCascade(const MatchResult&, CascadeLevel, OptionSet<PropertyType> includedProperties, const UncheckedKeyHashSet<AnimatableCSSProperty>* = nullptr);
+    static constexpr OptionSet<PropertyType> normalPropertyTypes() { return { PropertyType::NonInherited,  PropertyType::Inherited }; }
+    static constexpr OptionSet<PropertyType> startingStylePropertyTypes() { return normalPropertyTypes() | PropertyType::StartingStyle; }
+
+    struct IncludedProperties {
+        OptionSet<PropertyType> types;
+        // Ids are mutually exclusive with types. They are low-priority only and have any cascade aliases resolved.
+        Vector<CSSPropertyID, 4> ids { };
+
+        bool isEmpty() const { return !types && ids.isEmpty(); }
+    };
+
+    static IncludedProperties normalProperties() { return { normalPropertyTypes() }; }
+
+    PropertyCascade(const MatchResult&, CascadeLevel, IncludedProperties&&, const HashSet<AnimatableCSSProperty>* = nullptr, const StyleProperties* positionTryFallbackProperties = nullptr);
     PropertyCascade(const PropertyCascade&, CascadeLevel, std::optional<ScopeOrdinal> rollbackScope = { }, std::optional<CascadeLayerPriority> maximumCascadeLayerPriorityForRollback = { });
 
     ~PropertyCascade();
@@ -82,46 +93,51 @@ public:
     const Property& customProperty(const AtomString&) const;
 
     std::span<const CSSPropertyID> logicalGroupPropertyIDs() const;
-    const UncheckedKeyHashMap<AtomString, Property>& customProperties() const { return m_customProperties; }
+    const HashMap<AtomString, Property>& customProperties() const { return m_customProperties; }
 
-    const UncheckedKeyHashSet<AnimatableCSSProperty> overriddenAnimatedProperties() const;
+    const HashSet<AnimatableCSSProperty> overriddenAnimatedProperties() const;
 
     PropertyBitSet& propertyIsPresent() { return m_propertyIsPresent; }
     const PropertyBitSet& propertyIsPresent() const { return m_propertyIsPresent; }
 
+    bool applyLowPriorityOnly() const { return !m_includedProperties.ids.isEmpty(); }
+
 private:
     void buildCascade();
     bool addNormalMatches(CascadeLevel);
     void addImportantMatches(CascadeLevel);
     bool addMatch(const MatchedProperties&, CascadeLevel, IsImportant);
     bool shouldApplyAfterAnimation(const StyleProperties::PropertyReference&);
+    void addPositionTryFallbackProperties();
 
     void set(CSSPropertyID, CSSValue&, const MatchedProperties&, CascadeLevel);
     void setLogicalGroupProperty(CSSPropertyID, CSSValue&, const MatchedProperties&, CascadeLevel);
     static void setPropertyInternal(Property&, CSSPropertyID, CSSValue&, const MatchedProperties&, CascadeLevel);
 
     bool hasProperty(CSSPropertyID, const CSSValue&);
+    bool mayOverrideExistingProperty(CSSPropertyID, const CSSValue&);
 
     unsigned logicalGroupPropertyIndex(CSSPropertyID) const;
     void setLogicalGroupPropertyIndex(CSSPropertyID, unsigned);
     void sortLogicalGroupPropertyIDs();
 
     const MatchResult& m_matchResult;
-    const OptionSet<PropertyType> m_includedProperties;
+    const IncludedProperties m_includedProperties;
     const CascadeLevel m_maximumCascadeLevel;
     const std::optional<ScopeOrdinal> m_rollbackScope;
     const std::optional<CascadeLayerPriority> m_maximumCascadeLayerPriorityForRollback;
 
     struct AnimationLayer {
-        AnimationLayer(const UncheckedKeyHashSet<AnimatableCSSProperty>&);
+        AnimationLayer(const HashSet<AnimatableCSSProperty>&);
 
-        const UncheckedKeyHashSet<AnimatableCSSProperty>& properties;
-        UncheckedKeyHashSet<AnimatableCSSProperty> overriddenProperties;
+        const HashSet<AnimatableCSSProperty>& properties;
+        HashSet<AnimatableCSSProperty> overriddenProperties;
         bool hasCustomProperties { false };
         bool hasFontSize { false };
         bool hasLineHeight { false };
     };
     std::optional<AnimationLayer> m_animationLayer;
+    std::optional<MatchedProperties> m_positionTryFallbackProperties;
 
     // The CSSPropertyID enum is sorted like this:
     // 1. CSSPropertyInvalid and CSSPropertyCustom.
@@ -143,7 +159,7 @@ private:
     CSSPropertyID m_lowestSeenLogicalGroupProperty { lastLogicalGroupProperty };
     CSSPropertyID m_highestSeenLogicalGroupProperty { firstLogicalGroupProperty };
 
-    UncheckedKeyHashMap<AtomString, Property> m_customProperties;
+    HashMap<AtomString, Property> m_customProperties;
 };
 
 inline bool PropertyCascade::hasNormalProperty(CSSPropertyID id) const
