@@ -42,19 +42,6 @@ function nullish(value)
     return value === null || value === undefined;
 }
 
-Object.defineProperty(Object, "shallowCopy",
-{
-    value(object)
-    {
-        // Make a new object and copy all the key/values. The values are not copied.
-        var copy = {};
-        var keys = Object.keys(object);
-        for (var i = 0; i < keys.length; ++i)
-            copy[keys[i]] = object[keys[i]];
-        return copy;
-    }
-});
-
 Object.defineProperty(Object, "shallowEqual",
 {
     value(a, b)
@@ -171,6 +158,22 @@ Object.defineProperty(Map.prototype, "firstKey",
     }
 });
 
+Object.defineProperty(Map.prototype, "firstValue",
+{
+    get()
+    {
+        return this.values().next().value;
+    }
+});
+
+Object.defineProperty(Map.prototype, "lastKey",
+{
+    get()
+    {
+        return Array.from(this.keys()).lastValue;
+    }
+});
+
 Object.defineProperty(WeakMap.prototype, "getOrInitialize",
 {
     value(key, initialValue)
@@ -258,23 +261,6 @@ Object.defineProperty(Set.prototype, "equals",
     }
 });
 
-Object.defineProperty(Set.prototype, "difference",
-{
-    value(other)
-    {
-        if (other === this)
-            return new Set;
-
-        let result = new Set;
-        for (let item of this) {
-            if (!other.has(item))
-                result.add(item);
-        }
-
-        return result;
-    }
-});
-
 Object.defineProperty(Set.prototype, "firstValue",
 {
     get()
@@ -291,35 +277,6 @@ Object.defineProperty(Set.prototype, "lastValue",
     }
 });
 
-Object.defineProperty(Set.prototype, "intersects",
-{
-    value(other)
-    {
-        if (!this.size || !other.size)
-            return false;
-
-        for (let item of this) {
-            if (other.has(item))
-                return true;
-        }
-
-        return false;
-    }
-});
-
-Object.defineProperty(Set.prototype, "isSubsetOf",
-{
-    value(other)
-    {
-        for (let item of this) {
-            if (!other.has(item))
-                return false;
-        }
-
-        return true;
-    }
-});
-
 Object.defineProperty(Node.prototype, "traverseNextNode",
 {
     value(stayWithin)
