@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Igalia S.L.
+ * Copyright (C) 2023, 2025 Igalia S.L.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,18 +26,27 @@
 #include "config.h"
 #include "WPEViewWayland.h"
 
+#include "GRefPtrWPE.h"
 #include "WPEBufferDMABufFormats.h"
 #include "WPEDisplayWaylandPrivate.h"
 #include "WPEToplevelWaylandPrivate.h"
 #include "WPEWaylandSHMPool.h"
 #include "linux-dmabuf-unstable-v1-client-protocol.h"
 #include "pointer-constraints-unstable-v1-client-protocol.h"
+#if USE(SYSPROF_CAPTURE)
+#include "presentation-time-client-protocol.h"
+#endif
 #include "relative-pointer-unstable-v1-client-protocol.h"
 #include <gio/gio.h>
+#include <wtf/Deque.h>
 #include <wtf/FastMalloc.h>
+#include <wtf/MonotonicTime.h>
+#include <wtf/SystemTracing.h>
 #include <wtf/Vector.h>
 #include <wtf/glib/GRefPtr.h>
+#include <wtf/glib/GSpanExtras.h>
 #include <wtf/glib/GUniquePtr.h>
+#include <wtf/glib/GWeakPtr.h>
 #include <wtf/glib/WTFGType.h>
 
 // These includes need to be in this order because wayland-egl.h defines WL_EGL_PLATFORM
@@ -50,6 +59,29 @@
 typedef struct wl_buffer *(EGLAPIENTRYP PFNEGLCREATEWAYLANDBUFFERFROMIMAGEWL)(EGLDisplay dpy, EGLImageKHR image);
 #endif
 
+#if USE(SYSPROF_CAPTURE)
+static constexpr unsigned kFrameHistorySize = 30;
+
+class PresentationFeedbackStatistics {
+    WTF_DEPRECATED_MAKE_FAST_ALLOCATED(PresentationFeedbackStatistics);
+    WTF_MAKE_NONCOPYABLE(PresentationFeedbackStatistics);
+public:
+    explicit PresentationFeedbackStatistics(unsigned historySize);
+    ~PresentationFeedbackStatistics();
+
+    void beginFrame(struct wp_presentation*, WPEView*);
+    void presentedFrame(struct wp_presentation_feedback*, Seconds presentationTime, Seconds refreshInterval, uint64_t sequence, uint32_t flags);
+    void discardedFrame(struct wp_presentation_feedback*);
+
+    Seconds calculateAverageLatency() const;
+
+private:
+    unsigned m_maxHistorySize { 0 };
+    Vector<std::tuple<struct wp_presentation_feedback*, MonotonicTime>, 3> m_pendingFeedbacks;
+    Deque<Seconds> m_latencyHistory;
+};
+#endif
+
 /**
  * WPEViewWayland:
  *
@@ -58,6 +90,10 @@ struct _WPEViewWaylandPrivate {
     GRefPtr<WPEBuffer> buffer;
     struct wl_callback* frameCallback;
 
+#if USE(SYSPROF_CAPTURE)
+    std::unique_ptr<PresentationFeedbackStatistics> presentationFeedbackStatistics;
+#endif
+
     Vector<WPERectangle, 1> opaqueRegion;
     unsigned long resizedID;
 
@@ -82,7 +118,7 @@ static void wpeViewWaylandConstructed(GObject* object)
         priv->opaqueRegion.clear();
         priv->opaqueRegion.append({ 0, 0, wpe_view_get_width(view), wpe_view_get_height(view) });
         if (auto* toplevel = wpe_view_get_toplevel(view))
-            wpeToplevelWaylandSetOpaqueRectangles(WPE_TOPLEVEL_WAYLAND(toplevel), priv->opaqueRegion.data(), priv->opaqueRegion.size());
+            wpeToplevelWaylandSetOpaqueRectangles(WPE_TOPLEVEL_WAYLAND(toplevel), priv->opaqueRegion.mutableSpan().data(), priv->opaqueRegion.size());
     }), nullptr);
 
     g_signal_connect(view, "notify::toplevel", G_CALLBACK(+[](WPEView* view, GParamSpec*, gpointer) {
@@ -99,7 +135,7 @@ static void wpeViewWaylandConstructed(GObject* object)
             wpe_view_resized(view, width, height);
 
         auto* priv = WPE_VIEW_WAYLAND(view)->priv;
-        wpeToplevelWaylandSetOpaqueRectangles(WPE_TOPLEVEL_WAYLAND(toplevel), !priv->opaqueRegion.isEmpty() ? priv->opaqueRegion.data() : nullptr, priv->opaqueRegion.size());
+        wpeToplevelWaylandSetOpaqueRectangles(WPE_TOPLEVEL_WAYLAND(toplevel), !priv->opaqueRegion.isEmpty() ? priv->opaqueRegion.mutableSpan().data() : nullptr, priv->opaqueRegion.size());
 
         wpe_view_map(view);
     }), nullptr);
@@ -127,40 +163,96 @@ static void wpeViewWaylandDispose(GObject* object)
     g_clear_pointer(&priv->lockedPointer, zwp_locked_pointer_v1_destroy);
     g_clear_pointer(&priv->relativePointer, zwp_relative_pointer_v1_destroy);
 
+#if USE(SYSPROF_CAPTURE)
+    priv->presentationFeedbackStatistics = nullptr;
+#endif
+
     G_OBJECT_CLASS(wpe_view_wayland_parent_class)->dispose(object);
 }
 
-struct DMABufBuffer {
-    WTF_MAKE_STRUCT_FAST_ALLOCATED;
+class WaylandBuffer {
+    WTF_DEPRECATED_MAKE_FAST_ALLOCATED(WaylandBuffer);
+public:
+    virtual ~WaylandBuffer()
+    {
+        if (m_wlBuffer)
+            wl_buffer_destroy(m_wlBuffer);
+    }
 
-    explicit DMABufBuffer(wl_buffer* buffer)
-        : wlBuffer(buffer)
+    WPEView* view() const  { return m_view.get(); }
+    struct wl_buffer* wlBuffer() const { return m_wlBuffer; }
+
+protected:
+    WaylandBuffer(WPEView* view, wl_buffer* buffer)
+        : m_view(view)
+        , m_wlBuffer(buffer)
     {
     }
 
-    ~DMABufBuffer()
-    {
-        if (wlBuffer)
-            wl_buffer_destroy(wlBuffer);
-        if (release)
-            zwp_linux_buffer_release_v1_destroy(release);
-    }
-
-    struct wl_buffer* wlBuffer { nullptr };
-    struct zwp_linux_buffer_release_v1* release { nullptr };
+    GWeakPtr<WPEView> m_view;
+    struct wl_buffer* m_wlBuffer { nullptr };
 };
 
-static void dmaBufBufferDestroy(DMABufBuffer* buffer)
+static void waylandBufferDestroy(WaylandBuffer* buffer)
 {
     delete buffer;
 }
 
+class DMABufBuffer final : public WaylandBuffer {
+public:
+    DMABufBuffer(WPEView* view, wl_buffer* buffer)
+        : WaylandBuffer(view, buffer)
+    {
+    }
+
+    virtual ~DMABufBuffer()
+    {
+        if (m_release)
+            zwp_linux_buffer_release_v1_destroy(m_release);
+    }
+
+    void setRelease(struct zwp_linux_buffer_release_v1* release)
+    {
+        released();
+        m_release = release;
+    }
+
+    void released()
+    {
+        g_clear_pointer(&m_release, zwp_linux_buffer_release_v1_destroy);
+    }
+
+private:
+    struct zwp_linux_buffer_release_v1* m_release { nullptr };
+};
+
+class SharedMemoryBuffer final : public WaylandBuffer {
+public:
+    SharedMemoryBuffer(WPEView* view, std::unique_ptr<WPE::WaylandSHMPool>&& pool, uint32_t offset, uint32_t width, uint32_t height, uint32_t stride)
+        : WaylandBuffer(view, pool->createBuffer(offset, width, height, stride))
+        , m_wlPool(WTFMove(pool))
+    {
+    }
+
+    virtual ~SharedMemoryBuffer() = default;
+
+    WPE::WaylandSHMPool* wlPool() const { return m_wlPool.get(); }
+
+private:
+    std::unique_ptr<WPE::WaylandSHMPool> m_wlPool;
+};
+
 static const struct wl_buffer_listener bufferListener = {
     // release
     [](void* userData, struct wl_buffer*)
     {
         auto* buffer = WPE_BUFFER(userData);
-        wpe_view_buffer_released(wpe_buffer_get_view(buffer), buffer);
+        auto* waylandBuffer = static_cast<WaylandBuffer*>(wpe_buffer_get_user_data(buffer));
+        if (!waylandBuffer)
+            return;
+
+        if (auto* view = waylandBuffer->view())
+            wpe_view_buffer_released(view, buffer);
     }
 };
 
@@ -190,7 +282,7 @@ static DMABufBuffer* createWaylandBufferFromEGLImage(WPEView* view, WPEBuffer* b
     }
 
     if (auto* wlBuffer = s_eglCreateWaylandBufferFromImageWL(eglDisplay, eglImage))
-        return new DMABufBuffer(wlBuffer);
+        return new DMABufBuffer(view, wlBuffer);
 
     g_set_error(error, WPE_VIEW_ERROR, WPE_VIEW_ERROR_RENDER_FAILED, "Failed to render buffer: eglCreateWaylandBufferFromImageWL failed with error %#04x", eglGetError());
     return nullptr;
@@ -199,7 +291,7 @@ static DMABufBuffer* createWaylandBufferFromEGLImage(WPEView* view, WPEBuffer* b
 static struct wl_buffer* createWaylandBufferFromDMABuf(WPEView* view, WPEBuffer* buffer, GError** error)
 {
     if (auto* dmaBufBuffer = static_cast<DMABufBuffer*>(wpe_buffer_get_user_data(buffer)))
-        return dmaBufBuffer->wlBuffer;
+        return dmaBufBuffer->wlBuffer();
 
     auto* bufferDMABuf = WPE_BUFFER_DMA_BUF(buffer);
     DMABufBuffer* dmaBufBuffer = nullptr;
@@ -220,7 +312,7 @@ static struct wl_buffer* createWaylandBufferFromDMABuf(WPEView* view, WPEBuffer*
             return nullptr;
         }
 
-        dmaBufBuffer = new DMABufBuffer(wlBuffer);
+        dmaBufBuffer = new DMABufBuffer(view, wlBuffer);
     } else {
         dmaBufBuffer = createWaylandBufferFromEGLImage(view, buffer, error);
         if (!dmaBufBuffer)
@@ -229,38 +321,15 @@ static struct wl_buffer* createWaylandBufferFromDMABuf(WPEView* view, WPEBuffer*
 
     auto* toplevel = wpe_view_get_toplevel(WPE_VIEW(view));
     if (!wpeToplevelWaylandGetSurfaceSync(WPE_TOPLEVEL_WAYLAND(toplevel)) || wpe_buffer_dma_buf_get_rendering_fence(bufferDMABuf) == -1)
-        wl_buffer_add_listener(dmaBufBuffer->wlBuffer, &bufferListener, buffer);
+        wl_buffer_add_listener(dmaBufBuffer->wlBuffer(), &bufferListener, buffer);
 
-    wpe_buffer_set_user_data(buffer, dmaBufBuffer, reinterpret_cast<GDestroyNotify>(dmaBufBufferDestroy));
-    return dmaBufBuffer->wlBuffer;
+    wpe_buffer_set_user_data(buffer, dmaBufBuffer, reinterpret_cast<GDestroyNotify>(waylandBufferDestroy));
+    return dmaBufBuffer->wlBuffer();
 }
 
-struct SharedMemoryBuffer {
-    WTF_MAKE_STRUCT_FAST_ALLOCATED;
-
-    SharedMemoryBuffer(std::unique_ptr<WPE::WaylandSHMPool>&& pool, uint32_t offset, uint32_t width, uint32_t height, uint32_t stride)
-        : wlPool(WTFMove(pool))
-        , wlBuffer(wlPool->createBuffer(offset, width, height, stride))
-    {
-    }
-
-    ~SharedMemoryBuffer()
-    {
-        if (wlBuffer)
-            wl_buffer_destroy(wlBuffer);
-    }
-
-    std::unique_ptr<WPE::WaylandSHMPool> wlPool;
-    struct wl_buffer* wlBuffer;
-};
-
-static void sharedMemoryBufferDestroy(SharedMemoryBuffer* buffer)
-{
-    delete buffer;
-}
-
-static SharedMemoryBuffer* sharedMemoryBufferCreate(WPEDisplayWayland* display, GBytes* bytes, int width, int height, unsigned stride)
+static SharedMemoryBuffer* sharedMemoryBufferCreate(WPEView* view, GBytes* bytes, int width, int height, unsigned stride)
 {
+    auto* display = WPE_DISPLAY_WAYLAND(wpe_view_get_display(view));
     auto size = g_bytes_get_size(bytes);
     auto wlPool = WPE::WaylandSHMPool::create(wpe_display_wayland_get_wl_shm(display), size);
     if (!wlPool)
@@ -270,17 +339,16 @@ static SharedMemoryBuffer* sharedMemoryBufferCreate(WPEDisplayWayland* display,
     if (offset < 0)
         return nullptr;
 
-    memcpy(reinterpret_cast<char*>(wlPool->data()) + offset, g_bytes_get_data(bytes, nullptr), size);
-
-    return new SharedMemoryBuffer(WTFMove(wlPool), offset, width, height, stride);
+    wlPool->write(WTF::span(bytes), offset);
+    return new SharedMemoryBuffer(view, WTFMove(wlPool), offset, width, height, stride);
 }
 
 static struct wl_buffer* createWaylandBufferSHM(WPEView* view, WPEBuffer* buffer, GError** error)
 {
     if (auto* sharedMemoryBuffer = static_cast<SharedMemoryBuffer*>(wpe_buffer_get_user_data(buffer))) {
         GBytes* bytes = wpe_buffer_shm_get_data(WPE_BUFFER_SHM(buffer));
-        memcpy(reinterpret_cast<char*>(sharedMemoryBuffer->wlPool->data()), g_bytes_get_data(bytes, nullptr), sharedMemoryBuffer->wlPool->size());
-        return sharedMemoryBuffer->wlBuffer;
+        sharedMemoryBuffer->wlPool()->write(WTF::span(bytes));
+        return sharedMemoryBuffer->wlBuffer();
     }
 
     auto* bufferSHM = WPE_BUFFER_SHM(buffer);
@@ -289,18 +357,17 @@ static struct wl_buffer* createWaylandBufferSHM(WPEView* view, WPEBuffer* buffer
         return nullptr;
     }
 
-    auto* display = WPE_DISPLAY_WAYLAND(wpe_view_get_display(view));
-    auto* sharedMemoryBuffer = sharedMemoryBufferCreate(display, wpe_buffer_shm_get_data(bufferSHM),
+    auto* sharedMemoryBuffer = sharedMemoryBufferCreate(view, wpe_buffer_shm_get_data(bufferSHM),
         wpe_buffer_get_width(buffer), wpe_buffer_get_height(buffer), wpe_buffer_shm_get_stride(bufferSHM));
     if (!sharedMemoryBuffer) {
         g_set_error_literal(error, WPE_VIEW_ERROR, WPE_VIEW_ERROR_RENDER_FAILED, "Failed to render buffer: can't create Wayland buffer because failed to create shared memory");
         return nullptr;
     }
 
-    wl_buffer_add_listener(sharedMemoryBuffer->wlBuffer, &bufferListener, buffer);
+    wl_buffer_add_listener(sharedMemoryBuffer->wlBuffer(), &bufferListener, buffer);
 
-    wpe_buffer_set_user_data(buffer, sharedMemoryBuffer, reinterpret_cast<GDestroyNotify>(sharedMemoryBufferDestroy));
-    return sharedMemoryBuffer->wlBuffer;
+    wpe_buffer_set_user_data(buffer, sharedMemoryBuffer, reinterpret_cast<GDestroyNotify>(waylandBufferDestroy));
+    return sharedMemoryBuffer->wlBuffer();
 }
 
 static struct wl_buffer* createWaylandBuffer(WPEView* view, WPEBuffer* buffer, GError** error)
@@ -330,11 +397,138 @@ const struct wl_callback_listener frameListener = {
     }
 };
 
+#if USE(SYSPROF_CAPTURE)
+static const struct wp_presentation_feedback_listener presentationFeedbackListener = {
+    // sync_output
+    [](void*, struct wp_presentation_feedback*, struct wl_output*) {
+        // Quoting https://wayland.app/protocols/presentation-time#wp_presentation_feedback:event:presented:
+        // As presentation can be synchronized to only one output at a time, this event tells which output it was.
+        // This event is only sent prior to the presented event.
+    },
+    // presented
+    [](void* data, struct wp_presentation_feedback* feedback, uint32_t tvSecHi, uint32_t tvSecLo, uint32_t tvNsec, uint32_t refreshNsec, uint32_t seqHi, uint32_t seqLo, uint32_t flags) {
+        auto* priv = WPE_VIEW_WAYLAND(data)->priv;
+
+        auto combineToUInt64 = [](uint32_t lo, uint32_t hi) -> uint64_t {
+            return (static_cast<uint64_t>(hi) << 32) | lo;
+        };
+
+        auto parseTimeValue = [&](uint32_t secLo, uint32_t secHi, uint32_t nsec) -> Seconds {
+            return Seconds { static_cast<double>(combineToUInt64(secLo, secHi)) } + Seconds::fromNanoseconds(nsec);
+        };
+
+        // Quoting https://wayland.app/protocols/presentation-time#wp_presentation_feedback:event:presented:
+        // The associated content update was displayed to the user at the indicated time (tvSecHi/Lo, tvNsec).
+        // For the interpretation of the timestamp, see presentation.clock_id event.
+        auto presentationTime = parseTimeValue(tvSecLo, tvSecHi, tvNsec);
+        auto refreshInterval = Seconds::fromNanoseconds(refreshNsec);
+        auto sequence = combineToUInt64(seqLo, seqHi);
+        priv->presentationFeedbackStatistics->presentedFrame(feedback, presentationTime, refreshInterval, sequence, flags);
+    },
+    // discarded
+    [](void* data, struct wp_presentation_feedback* feedback) {
+        auto* priv = WPE_VIEW_WAYLAND(data)->priv;
+
+        // Quoting https://wayland.app/protocols/presentation-time#wp_presentation_feedback:event:discarded:
+        // The content update was never displayed to the user.
+        priv->presentationFeedbackStatistics->discardedFrame(feedback);
+    },
+};
+// END
+
+PresentationFeedbackStatistics::PresentationFeedbackStatistics(unsigned historySize)
+    : m_maxHistorySize(historySize)
+{
+}
+
+PresentationFeedbackStatistics::~PresentationFeedbackStatistics()
+{
+    for (auto [presentationFeedback, frameStartTime] : m_pendingFeedbacks)
+        wp_presentation_feedback_destroy(presentationFeedback);
+}
+
+void PresentationFeedbackStatistics::beginFrame(struct wp_presentation* presentation, WPEView* view)
+{
+    auto* wlSurface = wpe_view_wayland_get_wl_surface(WPE_VIEW_WAYLAND(view));
+
+    auto* presentationFeedback = wp_presentation_feedback(presentation, wlSurface);
+    wp_presentation_feedback_add_listener(presentationFeedback, &presentationFeedbackListener, view);
+    m_pendingFeedbacks.append({ presentationFeedback, MonotonicTime::now() });
+
+    WTFEmitSignpost(this, WaylandFrameBegin, "pending feedbacks? %lu", m_pendingFeedbacks.size());
+}
+
+void PresentationFeedbackStatistics::presentedFrame(struct wp_presentation_feedback* feedback, Seconds presentationTime, Seconds refreshInterval, uint64_t sequence, uint32_t flags)
+{
+    Seconds frameStartTime;
+    bool found = m_pendingFeedbacks.removeFirstMatching([&](const auto& entry) {
+        auto [presentationFeedback, frameStartTimeForFeedback] = entry;
+        if (presentationFeedback == feedback) {
+            frameStartTime = frameStartTimeForFeedback.secondsSinceEpoch();
+            wp_presentation_feedback_destroy(feedback);
+            return true;
+        }
+        return false;
+    });
+
+    RELEASE_ASSERT(found);
+
+    auto latency = presentationTime - frameStartTime;
+    m_latencyHistory.append(latency);
+    if (m_latencyHistory.size() > m_maxHistorySize)
+        m_latencyHistory.removeFirst();
+
+    bool hasHardwareClockFlag = flags & WP_PRESENTATION_FEEDBACK_KIND_HW_CLOCK;
+    bool hasHardwareCompletionFlag = flags & WP_PRESENTATION_FEEDBACK_KIND_HW_COMPLETION;
+    bool hasVSyncFlag = flags & WP_PRESENTATION_FEEDBACK_KIND_VSYNC;
+    bool hasZeroCopyFlag = flags & WP_PRESENTATION_FEEDBACK_KIND_ZERO_COPY;
+
+    auto averageLatency = calculateAverageLatency();
+    WTFSetCounter(FrameLatency, static_cast<int>(std::round(averageLatency.milliseconds())));
+    WTFEmitSignpost(this, WaylandFramePresented,
+        "sequence=%lu, latency=%.3f ms, refreshInterval=%.3f ms, flags =%s%s%s%s, pending feedbacks? %lu (avg. latency %.3f ms)",
+        sequence, latency.milliseconds(), refreshInterval.milliseconds(),
+        hasHardwareClockFlag ? " [hw-clock]" : "",
+        hasHardwareCompletionFlag ? " [hw-completion]" : "",
+        hasVSyncFlag ? " [vsync]" : "",
+        hasZeroCopyFlag ? "  [zero-copy]" : "",
+        m_pendingFeedbacks.size(),
+        averageLatency.milliseconds());
+}
+
+void PresentationFeedbackStatistics::discardedFrame(struct wp_presentation_feedback* feedback)
+{
+    m_pendingFeedbacks.removeFirstMatching([&](const auto& entry) {
+        auto [presentationFeedback, frameStartTimeForFeedback] = entry;
+        if (presentationFeedback == feedback) {
+            wp_presentation_feedback_destroy(feedback);
+            return true;
+        }
+        return false;
+    });
+
+    WTFEmitSignpost(this, WaylandFrameDiscarded, "pending feedbacks? %lu", m_pendingFeedbacks.size());
+}
+
+Seconds PresentationFeedbackStatistics::calculateAverageLatency() const
+{
+    if (m_latencyHistory.isEmpty())
+        return Seconds { 1.0 / 60.0 };
+
+    auto total = std::accumulate(m_latencyHistory.begin(), m_latencyHistory.end(), Seconds { });
+    return total / m_latencyHistory.size();
+}
+#endif
+
 static void dmaBufBufferReleased(WPEBuffer* buffer)
 {
-    wpe_view_buffer_released(wpe_buffer_get_view(buffer), buffer);
-    if (auto* dmaBufBuffer = static_cast<DMABufBuffer*>(wpe_buffer_get_user_data(buffer)))
-        g_clear_pointer(&dmaBufBuffer->release, zwp_linux_buffer_release_v1_destroy);
+    auto* dmaBufBuffer = static_cast<DMABufBuffer*>(wpe_buffer_get_user_data(buffer));
+    if (!dmaBufBuffer)
+        return;
+
+    if (auto* view = dmaBufBuffer->view())
+        wpe_view_buffer_released(view, buffer);
+    dmaBufBuffer->released();
 }
 
 const struct zwp_linux_buffer_release_v1_listener bufferReleaseListener = {
@@ -358,10 +552,22 @@ static gboolean wpeViewWaylandRenderBuffer(WPEView* view, WPEBuffer* buffer, con
     if (!wlBuffer)
         return FALSE;
 
+    auto* toplevel = wpe_view_get_toplevel(view);
+    if (wpe_toplevel_get_state(toplevel) & WPE_TOPLEVEL_STATE_MAXIMIZED) {
+        // The surface is maximized. The window geometry specified in the configure
+        // event must be obeyed by the client, or the xdg_wm_base.invalid_surface_state
+        // error is raised.
+        auto scale = wpe_view_get_scale(view);
+        if (wpe_view_get_width(view) * scale != wpe_buffer_get_width(buffer) || wpe_view_get_height(view) * scale != wpe_buffer_get_height(buffer)) {
+            wpe_view_buffer_rendered(view, buffer);
+            return TRUE;
+        }
+    }
+
     auto* priv = WPE_VIEW_WAYLAND(view)->priv;
     priv->buffer = buffer;
 
-    wpeToplevelWaylandUpdateOpaqueRegion(WPE_TOPLEVEL_WAYLAND(wpe_view_get_toplevel(view)));
+    wpeToplevelWaylandUpdateOpaqueRegion(WPE_TOPLEVEL_WAYLAND(toplevel));
 
     auto* wlSurface = wpe_view_wayland_get_wl_surface(WPE_VIEW_WAYLAND(view));
     wl_surface_attach(wlSurface, wlBuffer, 0, 0);
@@ -372,25 +578,37 @@ static gboolean wpeViewWaylandRenderBuffer(WPEView* view, WPEBuffer* buffer, con
             auto* surfaceSync = wpeToplevelWaylandGetSurfaceSync(WPE_TOPLEVEL_WAYLAND(wpe_view_get_toplevel(view)));
             zwp_linux_surface_synchronization_v1_set_acquire_fence(surfaceSync, renderingFence.value());
 
+            auto* release = zwp_linux_surface_synchronization_v1_get_release(surfaceSync);
+            zwp_linux_buffer_release_v1_add_listener(release, &bufferReleaseListener, buffer);
             auto* dmaBufBuffer = static_cast<DMABufBuffer*>(wpe_buffer_get_user_data(buffer));
-            dmaBufBuffer->release = zwp_linux_surface_synchronization_v1_get_release(surfaceSync);
-            zwp_linux_buffer_release_v1_add_listener(dmaBufBuffer->release, &bufferReleaseListener, buffer);
+            dmaBufBuffer->setRelease(release);
         }
     }
 
     auto* display = WPE_DISPLAY_WAYLAND(wpe_view_get_display(view));
     auto* wlCompositor = wpe_display_wayland_get_wl_compositor(display);
-    if (nDamageRects && LIKELY(wl_compositor_get_version(wlCompositor) >= 4)) {
+    if (nDamageRects && wl_compositor_get_version(wlCompositor) >= 4) [[likely]] {
         ASSERT(damageRects);
         for (unsigned i = 0; i < nDamageRects; ++i)
             wl_surface_damage_buffer(wlSurface, damageRects[i].x, damageRects[i].y, damageRects[i].width, damageRects[i].height);
     } else
         wl_surface_damage(wlSurface, 0, 0, INT32_MAX, INT32_MAX);
 
+    RELEASE_ASSERT(!priv->frameCallback);
     priv->frameCallback = wl_surface_frame(wlSurface);
     wl_callback_add_listener(priv->frameCallback, &frameListener, view);
-    wl_surface_commit(wlSurface);
 
+#if USE(SYSPROF_CAPTURE)
+    if (auto* annotator = SysprofAnnotator::singletonIfCreated()) {
+        if (auto* presentation = wpeDisplayWaylandGetPresentation(display)) {
+            if (!priv->presentationFeedbackStatistics)
+                priv->presentationFeedbackStatistics = makeUnique<PresentationFeedbackStatistics>(kFrameHistorySize);
+            priv->presentationFeedbackStatistics->beginFrame(presentation, view);
+        }
+    }
+#endif
+
+    wl_surface_commit(wlSurface);
     return TRUE;
 }
 
@@ -404,7 +622,7 @@ static const struct wl_buffer_listener cursorBufferListener = {
     // release
     [](void* userData, struct wl_buffer*)
     {
-        sharedMemoryBufferDestroy(static_cast<SharedMemoryBuffer*>(userData));
+        waylandBufferDestroy(static_cast<WaylandBuffer*>(userData));
     }
 };
 
@@ -419,9 +637,8 @@ static const struct zwp_relative_pointer_v1_listener relativePointerListener = {
         double dX = wl_fixed_to_double(deltaX);
         double dY = wl_fixed_to_double(deltaY);
 
-        auto* event = wpe_event_pointer_move_new(WPE_EVENT_POINTER_MOVE, view, WPE_INPUT_SOURCE_MOUSE, 0, static_cast<WPEModifiers>(pointerModifiers), x, y, dX, dY);
-        wpe_view_event(view, event);
-        wpe_event_unref(event);
+        GRefPtr<WPEEvent> event = adoptGRef(wpe_event_pointer_move_new(WPE_EVENT_POINTER_MOVE, view, WPE_INPUT_SOURCE_MOUSE, 0, static_cast<WPEModifiers>(pointerModifiers), x, y, dX, dY));
+        wpe_view_event(view, event.get());
     }
 };
 
@@ -466,12 +683,12 @@ static void wpeViewWaylandSetCursorFromBytes(WPEView* view, GBytes* bytes, guint
     if (!cursor)
         return;
 
-    auto* sharedMemoryBuffer = sharedMemoryBufferCreate(display, bytes, width, height, stride);
+    auto* sharedMemoryBuffer = sharedMemoryBufferCreate(view, bytes, width, height, stride);
     if (!sharedMemoryBuffer)
         return;
 
-    wl_buffer_add_listener(sharedMemoryBuffer->wlBuffer, &cursorBufferListener, sharedMemoryBuffer);
-    cursor->setFromBuffer(sharedMemoryBuffer->wlBuffer, width, height, hotspotX, hotspotY);
+    wl_buffer_add_listener(sharedMemoryBuffer->wlBuffer(), &cursorBufferListener, sharedMemoryBuffer);
+    cursor->setFromBuffer(sharedMemoryBuffer->wlBuffer(), width, height, hotspotX, hotspotY);
 }
 
 static void wpeViewWaylandSetOpaqueRectangles(WPEView* view, WPERectangle* rects, guint rectsCount)
@@ -489,7 +706,7 @@ static void wpeViewWaylandSetOpaqueRectangles(WPEView* view, WPERectangle* rects
             priv->opaqueRegion.append(rects[i]);
     }
     if (auto* toplevel = wpe_view_get_toplevel(view))
-        wpeToplevelWaylandSetOpaqueRectangles(WPE_TOPLEVEL_WAYLAND(toplevel), !priv->opaqueRegion.isEmpty() ? priv->opaqueRegion.data() : nullptr, priv->opaqueRegion.size());
+        wpeToplevelWaylandSetOpaqueRectangles(WPE_TOPLEVEL_WAYLAND(toplevel), !priv->opaqueRegion.isEmpty() ? priv->opaqueRegion.mutableSpan().data() : nullptr, priv->opaqueRegion.size());
 }
 
 static gboolean wpeViewWaylandCanBeMapped(WPEView* view)
