@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2019-2020 Apple Inc. All rights reserved.
+* Copyright (C) 2019-2025 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
@@ -26,21 +26,19 @@
 #include "config.h"
 #include "WebResourceLoadObserver.h"
 
-#if ENABLE(INTELLIGENT_TRACKING_PREVENTION)
-
 #include "Logging.h"
 #include "NetworkConnectionToWebProcessMessages.h"
 #include "NetworkProcessConnection.h"
-#include "WebCoreArgumentCoders.h"
 #include "WebPage.h"
 #include "WebProcess.h"
-#include <WebCore/DeprecatedGlobalSettings.h>
-#include <WebCore/Frame.h>
+#include <WebCore/DocumentInlines.h>
 #include <WebCore/FrameDestructionObserverInlines.h>
 #include <WebCore/FrameLoader.h>
-#include <WebCore/FrameLoaderClient.h>
 #include <WebCore/HTMLFrameOwnerElement.h>
+#include <WebCore/LocalFrame.h>
+#include <WebCore/LocalFrameLoaderClient.h>
 #include <WebCore/Page.h>
+#include <wtf/text/MakeString.h>
 
 namespace WebKit {
 
@@ -81,7 +79,7 @@ void WebResourceLoadObserver::requestStorageAccessUnderOpener(const RegistrableD
     if (domainInNeedOfStorageAccess != openerDomain
         && !openerDocument.hasRequestedPageSpecificStorageAccessWithUserInteraction(domainInNeedOfStorageAccess)
         && !openerUrl.isAboutBlank()) {
-        WebProcess::singleton().ensureNetworkProcessConnection().connection().send(Messages::NetworkConnectionToWebProcess::RequestStorageAccessUnderOpener(domainInNeedOfStorageAccess, openerPage.identifier(), openerDomain), 0);
+        Ref { WebProcess::singleton().ensureNetworkProcessConnection().connection() }->send(Messages::NetworkConnectionToWebProcess::RequestStorageAccessUnderOpener(domainInNeedOfStorageAccess, openerPage.identifier(), openerDomain), 0);
         
         openerPage.addDomainWithPageLevelStorageAccess(openerDomain, domainInNeedOfStorageAccess);
 
@@ -113,7 +111,7 @@ void WebResourceLoadObserver::scheduleNotificationIfNeeded()
 void WebResourceLoadObserver::updateCentralStatisticsStore(CompletionHandler<void()>&& completionHandler)
 {
     m_notificationTimer.stop();
-    WebProcess::singleton().ensureNetworkProcessConnection().connection().sendWithAsyncReply(Messages::NetworkConnectionToWebProcess::ResourceLoadStatisticsUpdated(takeStatistics()), WTFMove(completionHandler));
+    Ref { WebProcess::singleton().ensureNetworkProcessConnection().connection() }->sendWithAsyncReply(Messages::NetworkConnectionToWebProcess::ResourceLoadStatisticsUpdated(takeStatistics()), WTFMove(completionHandler));
 }
 
 
@@ -123,7 +121,7 @@ String WebResourceLoadObserver::statisticsForURL(const URL& url)
     if (!statistics)
         return emptyString();
 
-    return makeString("Statistics for ", url.host().toString(), ":\n", statistics->toString());
+    return makeString("Statistics for "_s, url.host().toString(), ":\n"_s, statistics->toString());
 }
 
 Vector<ResourceLoadStatistics> WebResourceLoadObserver::takeStatistics()
@@ -150,6 +148,10 @@ void WebResourceLoadObserver::logFontLoad(const Document& document, const String
     if (isEphemeral())
         return;
 
+    RefPtr page = document.page();
+    if (!page)
+        return;
+
 #if ENABLE(WEB_API_STATISTICS)
     RegistrableDomain registrableDomain { document.url() };
     auto& statistics = ensureResourceStatisticsForRegistrableDomain(registrableDomain);
@@ -161,7 +163,7 @@ void WebResourceLoadObserver::logFontLoad(const Document& document, const String
         if (statistics.fontsSuccessfullyLoaded.add(familyName).isNewEntry)
             shouldCallNotificationCallback = true;
     }
-    RegistrableDomain mainFrameRegistrableDomain { document.topDocument().url() };
+    RegistrableDomain mainFrameRegistrableDomain { page->mainFrameURL() };
     if (statistics.topFrameRegistrableDomainsWhichAccessedWebAPIs.add(mainFrameRegistrableDomain).isNewEntry)
         shouldCallNotificationCallback = true;
     if (shouldCallNotificationCallback)
@@ -178,10 +180,14 @@ void WebResourceLoadObserver::logCanvasRead(const Document& document)
     if (isEphemeral())
         return;
 
+    RefPtr page = document.page();
+    if (!page)
+        return;
+
 #if ENABLE(WEB_API_STATISTICS)
     RegistrableDomain registrableDomain { document.url() };
     auto& statistics = ensureResourceStatisticsForRegistrableDomain(registrableDomain);
-    RegistrableDomain mainFrameRegistrableDomain { document.topDocument().url() };
+    RegistrableDomain mainFrameRegistrableDomain { page->mainFrameURL() };
     statistics.canvasActivityRecord.wasDataRead = true;
     if (statistics.topFrameRegistrableDomainsWhichAccessedWebAPIs.add(mainFrameRegistrableDomain).isNewEntry)
         scheduleNotificationIfNeeded();
@@ -195,11 +201,15 @@ void WebResourceLoadObserver::logCanvasWriteOrMeasure(const Document& document,
     if (isEphemeral())
         return;
 
+    RefPtr page = document.page();
+    if (!page)
+        return;
+
 #if ENABLE(WEB_API_STATISTICS)
     RegistrableDomain registrableDomain { document.url() };
     auto& statistics = ensureResourceStatisticsForRegistrableDomain(registrableDomain);
     bool shouldCallNotificationCallback = false;
-    RegistrableDomain mainFrameRegistrableDomain { document.topDocument().url() };
+    RegistrableDomain mainFrameRegistrableDomain { page->mainFrameURL() };
     if (statistics.canvasActivityRecord.recordWrittenOrMeasuredText(textWritten))
         shouldCallNotificationCallback = true;
     if (statistics.topFrameRegistrableDomainsWhichAccessedWebAPIs.add(mainFrameRegistrableDomain).isNewEntry)
@@ -212,11 +222,15 @@ void WebResourceLoadObserver::logCanvasWriteOrMeasure(const Document& document,
 #endif
 }
     
-void WebResourceLoadObserver::logNavigatorAPIAccessed(const Document& document, const ResourceLoadStatistics::NavigatorAPI functionName)
+void WebResourceLoadObserver::logNavigatorAPIAccessed(const Document& document, const NavigatorAPIsAccessed functionName)
 {
     if (isEphemeral())
         return;
 
+    RefPtr page = document.page();
+    if (!page)
+        return;
+
 #if ENABLE(WEB_API_STATISTICS)
     RegistrableDomain registrableDomain { document.url() };
     auto& statistics = ensureResourceStatisticsForRegistrableDomain(registrableDomain);
@@ -225,7 +239,7 @@ void WebResourceLoadObserver::logNavigatorAPIAccessed(const Document& document,
         statistics.navigatorFunctionsAccessed.add(functionName);
         shouldCallNotificationCallback = true;
     }
-    RegistrableDomain mainFrameRegistrableDomain { document.topDocument().url() };
+    RegistrableDomain mainFrameRegistrableDomain { page->mainFrameURL() };
     if (statistics.topFrameRegistrableDomainsWhichAccessedWebAPIs.add(mainFrameRegistrableDomain).isNewEntry)
         shouldCallNotificationCallback = true;
     if (shouldCallNotificationCallback)
@@ -236,11 +250,15 @@ void WebResourceLoadObserver::logNavigatorAPIAccessed(const Document& document,
 #endif
 }
     
-void WebResourceLoadObserver::logScreenAPIAccessed(const Document& document, const ResourceLoadStatistics::ScreenAPI functionName)
+void WebResourceLoadObserver::logScreenAPIAccessed(const Document& document, const ScreenAPIsAccessed functionName)
 {
     if (isEphemeral())
         return;
 
+    RefPtr page = document.page();
+    if (!page)
+        return;
+
 #if ENABLE(WEB_API_STATISTICS)
     RegistrableDomain registrableDomain { document.url() };
     auto& statistics = ensureResourceStatisticsForRegistrableDomain(registrableDomain);
@@ -249,7 +267,7 @@ void WebResourceLoadObserver::logScreenAPIAccessed(const Document& document, con
         statistics.screenFunctionsAccessed.add(functionName);
         shouldCallNotificationCallback = true;
     }
-    RegistrableDomain mainFrameRegistrableDomain { document.topDocument().url() };
+    RegistrableDomain mainFrameRegistrableDomain { page->mainFrameURL() };
     if (statistics.topFrameRegistrableDomainsWhichAccessedWebAPIs.add(mainFrameRegistrableDomain).isNewEntry)
         shouldCallNotificationCallback = true;
     if (shouldCallNotificationCallback)
@@ -260,7 +278,7 @@ void WebResourceLoadObserver::logScreenAPIAccessed(const Document& document, con
 #endif
 }
 
-void WebResourceLoadObserver::logSubresourceLoading(const Frame* frame, const ResourceRequest& newRequest, const ResourceResponse& redirectResponse, FetchDestinationIsScriptLike isScriptLike)
+void WebResourceLoadObserver::logSubresourceLoading(const LocalFrame* frame, const ResourceRequest& newRequest, const ResourceResponse& redirectResponse, FetchDestinationIsScriptLike isScriptLike)
 {
     if (isEphemeral())
         return;
@@ -270,15 +288,15 @@ void WebResourceLoadObserver::logSubresourceLoading(const Frame* frame, const Re
     if (!frame)
         return;
 
-    auto* page = frame->page();
+    RefPtr page = frame->page();
     if (!page)
         return;
+    const URL& topFrameURL = page->mainFrameURL();
 
     bool isRedirect = is3xxRedirect(redirectResponse);
     const URL& redirectedFromURL = redirectResponse.url();
     const URL& targetURL = newRequest.url();
-    const URL& topFrameURL = frame ? frame->mainFrame().document()->url() : URL();
-    
+
     auto targetHost = targetURL.host();
     auto topFrameHost = topFrameURL.host();
 
@@ -347,7 +365,7 @@ void WebResourceLoadObserver::logWebSocketLoading(const URL& targetURL, const UR
 void WebResourceLoadObserver::logUserInteractionWithReducedTimeResolution(const Document& document)
 {
     auto& url = document.url();
-    if (url.protocolIsAbout() || url.isLocalFile() || url.isEmpty())
+    if (url.protocolIsAbout() || url.protocolIsFile() || url.isEmpty())
         return;
 
     RegistrableDomain topFrameDomain { url };
@@ -365,18 +383,18 @@ void WebResourceLoadObserver::logUserInteractionWithReducedTimeResolution(const
         statistics.mostRecentUserInteractionTime = newTime;
     }
 
-    if (auto* frame = document.frame()) {
-        if (auto* opener = frame->loader().opener()) {
-            if (auto* openerDocument = opener->document()) {
-                if (auto* openerPage = openerDocument->page())
-                    requestStorageAccessUnderOpener(topFrameDomain, WebPage::fromCorePage(*openerPage), *openerDocument);
+    if (RefPtr frame = document.frame()) {
+        if (RefPtr opener = dynamicDowncast<LocalFrame>(frame->opener())) {
+            if (RefPtr openerDocument = opener->document()) {
+                if (RefPtr openerPage = openerDocument->page())
+                    requestStorageAccessUnderOpener(topFrameDomain, Ref { *WebPage::fromCorePage(*openerPage) }, *openerDocument);
             }
         }
     }
 
     // We notify right away in case of a user interaction instead of waiting the usual 5 seconds because we want
     // to update cookie blocking state as quickly as possible.
-    WebProcess::singleton().ensureNetworkProcessConnection().connection().send(Messages::NetworkConnectionToWebProcess::LogUserInteraction(topFrameDomain), 0);
+    Ref { WebProcess::singleton().ensureNetworkProcessConnection().connection() }->send(Messages::NetworkConnectionToWebProcess::LogUserInteraction(topFrameDomain), 0);
 
 #if !RELEASE_LOG_DISABLED
     if (shouldLogUserInteraction) {
@@ -392,9 +410,9 @@ void WebResourceLoadObserver::logUserInteractionWithReducedTimeResolution(const
         auto escapedURL = escapeForJSON(url.string());
         auto escapedDomain = escapeForJSON(topFrameDomain.string());
 
-        LOCAL_LOG(R"({ "url": "%{public}s",)", escapedURL.utf8().data());
-        LOCAL_LOG(R"(  "domain" : "%{public}s",)", escapedDomain.utf8().data());
-        LOCAL_LOG(R"(  "until" : %f })", newTime.secondsSinceEpoch().seconds());
+        LOCAL_LOG("{ \"url\": \"%" PUBLIC_LOG_STRING "\",", escapedURL.utf8().data());
+        LOCAL_LOG("  \"domain\" : \"%" PUBLIC_LOG_STRING "\",", escapedDomain.utf8().data());
+        LOCAL_LOG("  \"until\" : %f }", newTime.secondsSinceEpoch().seconds());
 
 #undef LOCAL_LOG
     }
@@ -427,21 +445,23 @@ bool WebResourceLoadObserver::hasCrossPageStorageAccess(const SubFrameDomain& su
     return false;
 }
 
-void WebResourceLoadObserver::setDomainsWithCrossPageStorageAccess(HashMap<TopFrameDomain, SubFrameDomain>&& domains, CompletionHandler<void()>&& completionHandler)
+void WebResourceLoadObserver::setDomainsWithCrossPageStorageAccess(HashMap<TopFrameDomain, Vector<SubFrameDomain>>&& domains, CompletionHandler<void()>&& completionHandler)
 {
-    for (auto& topDomain : domains.keys()) {
-        m_domainsWithCrossPageStorageAccess.ensure(topDomain, [] { return HashSet<RegistrableDomain> { };
-            }).iterator->value.add(domains.get(topDomain));
+    for (const auto& [topDomain, subResourceDomains] : domains) {
+        for (auto& subResourceDomain : subResourceDomains) {
+            m_domainsWithCrossPageStorageAccess.ensure(topDomain, [] {
+                return HashSet<RegistrableDomain> { };
+            }).iterator->value.add(subResourceDomain);
 
-        // Some sites have quirks where multiple login domains require storage access.
-        if (auto additionalLoginDomain = WebCore::NetworkStorageSession::findAdditionalLoginDomain(topDomain, domains.get(topDomain))) {
-            m_domainsWithCrossPageStorageAccess.ensure(topDomain, [] { return HashSet<RegistrableDomain> { };
+            // Some sites have quirks where multiple login domains require storage access.
+            if (auto additionalLoginDomain = WebCore::NetworkStorageSession::findAdditionalLoginDomain(topDomain, subResourceDomain)) {
+                m_domainsWithCrossPageStorageAccess.ensure(topDomain, [] {
+                    return HashSet<RegistrableDomain> { };
                 }).iterator->value.add(*additionalLoginDomain);
+            }
         }
     }
     completionHandler();
 }
 
 } // namespace WebKit
-
-#endif // ENABLE(INTELLIGENT_TRACKING_PREVENTION)
