@@ -26,6 +26,8 @@
 #ifndef PAS_SEGREGATED_PAGE_H
 #define PAS_SEGREGATED_PAGE_H
 
+PAS_IGNORE_WARNINGS_BEGIN("cast-align")
+
 #include "pas_allocator_index.h"
 #include "pas_bitvector.h"
 #include "pas_config.h"
@@ -265,7 +267,7 @@ pas_segregated_page_number_of_objects(unsigned object_size,
                                       pas_segregated_page_config page_config,
                                       pas_segregated_page_role role)
 {
-    static const bool verbose = false;
+    static const bool verbose = PAS_SHOULD_LOG(PAS_LOG_SEGREGATED_HEAPS);
     unsigned result;
     
     result = pas_segregated_page_useful_object_payload_size(object_size, page_config, role) / object_size;
@@ -273,7 +275,7 @@ pas_segregated_page_number_of_objects(unsigned object_size,
     if (verbose) {
         pas_log("first object offset = %u\n", pas_segregated_page_offset_from_page_boundary_to_first_object_exclusive(object_size, page_config));
         pas_log("end of last object offset = %u\n", pas_segregated_page_offset_from_page_boundary_to_end_of_last_object_exclusive(object_size, page_config));
-        pas_log("payload offset = %lu\n", pas_segregated_page_config_payload_offset_for_role(page_config, role));
+        pas_log("payload offset = %zu\n", pas_segregated_page_config_payload_offset_for_role(page_config, role));
         pas_log("object_size = %u, so number_of_objects = %u\n", object_size, result);
     }
     
@@ -474,5 +476,7 @@ pas_segregated_page_for_address_and_heap_config(uintptr_t begin, const pas_heap_
 
 PAS_END_EXTERN_C;
 
+PAS_IGNORE_WARNINGS_END
+
 #endif /* PAS_SEGREGATED_PAGE_H */
 
